package com.dartmaker.uponevent;

import android.app.Activity;
import android.content.Context;
import android.graphics.Bitmap;
import android.graphics.BitmapFactory;
import android.graphics.Canvas;
import android.graphics.Point;
import android.net.Uri;
import android.util.DisplayMetrics;
import android.view.Display;
import android.view.LayoutInflater;
import android.view.View;
import android.view.WindowManager;
import android.view.inputmethod.InputMethodManager;
import android.widget.ImageView;

import com.google.android.gms.maps.CameraUpdateFactory;
import com.google.android.gms.maps.GoogleMap;
import com.google.android.gms.maps.model.BitmapDescriptorFactory;
import com.google.android.gms.maps.model.CameraPosition;
import com.google.android.gms.maps.model.LatLng;
import com.google.android.gms.maps.model.MarkerOptions;

import java.io.IOException;
import java.io.InputStream;
import java.security.Timestamp;
import java.util.Calendar;
import java.util.Date;
import java.util.Locale;

public class ApplicationUtils {

    //AQUESTA FUNCIO RETORNA UNA IMATGE BITMAP COMPRIMIDA A UN MAXIM DE 0,5 MP A PARTIR D'UNA URI
    public Bitmap getCompressedBitmap(Uri uri, Context context) {
        InputStream in;
        try {
            final int IMAGE_MAX_SIZE = 500000; // 0,5MP
            in = context.getContentResolver().openInputStream(uri);

            // DECODIFIQUEM LA MIDA DE LA IMATGE
            BitmapFactory.Options options = new BitmapFactory.Options();
            options.inJustDecodeBounds = true;
            BitmapFactory.decodeStream(in, null, options);
            in.close();


            int scale = 1;
            while ((options.outWidth * options.outHeight) * (1 / Math.pow(scale, 2)) > IMAGE_MAX_SIZE) {
                scale++;
            }

            Bitmap resultBitmap;
            in = context.getContentResolver().openInputStream(uri);
            if (scale > 1) {
                scale--;
                options = new BitmapFactory.Options();
                options.inSampleSize = scale;
                resultBitmap = BitmapFactory.decodeStream(in, null, options);

                //REDIMENSIONEM A LES MIDES NECESSARIS
                int height = resultBitmap.getHeight();
                int width = resultBitmap.getWidth();

                double y = Math.sqrt(IMAGE_MAX_SIZE
                        / (((double) width) / height));
                double x = (y / height) * width;

                Bitmap scaledBitmap = Bitmap.createScaledBitmap(resultBitmap, (int) x,
                        (int) y, true);
                resultBitmap.recycle();
                resultBitmap = scaledBitmap;

                System.gc();
            } else {
                resultBitmap = BitmapFactory.decodeStream(in);
            }
            in.close();

            return resultBitmap;
        } catch (IOException e) {
            return null;
        }
    }

    //FUNCIO PER CREAR UN MAPA ESTATIC DONADA UNA LATITUD I UNA LONGITUD
    public static GoogleMap configStaticMap(Activity act, GoogleMap googleMap, double latitude, double longitude) {
        googleMap.setMapType(GoogleMap.MAP_TYPE_NORMAL);
        googleMap.getUiSettings().setZoomControlsEnabled(false);
        googleMap.getUiSettings().setMyLocationButtonEnabled(false);
        googleMap.getUiSettings().setCompassEnabled(false);
        googleMap.getUiSettings().setRotateGesturesEnabled(false);
        googleMap.getUiSettings().setZoomGesturesEnabled(false);
        googleMap.isTrafficEnabled();
        googleMap.setTrafficEnabled(false);
        googleMap.getUiSettings().setScrollGesturesEnabled(false);
        googleMap.getUiSettings().setMapToolbarEnabled(false);

        LayoutInflater inflater = (LayoutInflater) act.getSystemService(Context.LAYOUT_INFLATER_SERVICE);
        View marker_view = inflater.inflate(R.layout.maps_marker, null);
        ((ImageView) marker_view.findViewById(R.id.marker_bg)).setColorFilter(act.getResources().getColor(R.color.marker_secondary));

        CameraPosition cameraPosition = new CameraPosition.Builder().target(new LatLng(latitude, longitude)).zoom(12).build();
        MarkerOptions markerOptions = new MarkerOptions().position(new LatLng(latitude, longitude));
        markerOptions.icon(BitmapDescriptorFactory.fromBitmap(createBitmapFromView(act, marker_view)));
        googleMap.addMarker(markerOptions);
        googleMap.moveCamera(CameraUpdateFactory.newCameraPosition(cameraPosition));
        return googleMap;
    }

    //FUNCIO PER RETORNAR UNA IMATGE DE BITMAP DONADA UNA VIEW
    public static Bitmap createBitmapFromView(Activity act, View view) {
        DisplayMetrics displayMetrics = new DisplayMetrics();
        act.getWindowManager().getDefaultDisplay().getMetrics(displayMetrics);

        view.measure(displayMetrics.widthPixels, displayMetrics.heightPixels);
        view.layout(0, 0, displayMetrics.widthPixels, displayMetrics.heightPixels);
        view.buildDrawingCache();
        Bitmap bitmap = Bitmap.createBitmap(view.getMeasuredWidth(), view.getMeasuredHeight(), Bitmap.Config.ARGB_8888);

        Canvas canvas = new Canvas(bitmap);
        view.draw(canvas);

        return bitmap;
    }

    //AQUESTA FUNCIO RETORNA UN BITMAP REDIMENSIONAT A LA MAXIMA AMPLADA DE LA PANTALLA I ALTURA PROPORCIONAL
    //(SEMPRE QUE NO EXCEDEIXI UNA ALTURA MAXIMA)
    // EN CAS D'EXCEDIR L'ALTURA MAXIMA RETORNA UN BITMAP REDIMENTIONAT A L'ALTURA MAXIMA I AMPLADA PROPORCIONAL
    public static Bitmap resizeFullWidthMaxHeight(Bitmap bitmap, int maxHeight, Context context) {
        WindowManager wm = (WindowManager) context.getSystemService(Context.WINDOW_SERVICE);
        Display display = wm.getDefaultDisplay();
        Point size = new Point();
        display.getSize(size);
        int displayWidth = size.x;
        int displayHeight = size.y;

        //EN CAS QUE ES DONI LA ALTURA EN PORCENTATGE CAL EMPRAR EL SEGÜENT CODI
        //float exactMaxHeight = ((float)maxHeightPercentage/(float)100)*(float)displayHeight;
        //int maxHeight = Math.round(exactMaxHeight);

        if (bitmap.getHeight() > maxHeight) {
            int destWidth = (bitmap.getWidth() * maxHeight) / bitmap.getHeight();
            Bitmap b2 = Bitmap.createScaledBitmap(bitmap, destWidth, maxHeight, false);
            return b2;

        } else {
            int destHeight = (bitmap.getHeight() * displayWidth) / bitmap.getWidth();
            Bitmap b2 = Bitmap.createScaledBitmap(bitmap, displayWidth, destHeight, false);
            return b2;
        }
    }

    //RETORNA UNA APROXIMACIO A LA DIFERENCIA EN DIES MESOS I ANYS
    public static String getDateDiff(long start, long end) {
        Calendar startCalendar = Calendar.getInstance();
        startCalendar.setTimeInMillis(start);

        Calendar endCalendar = Calendar.getInstance();
        endCalendar.setTimeInMillis(end);

        Date endDate = endCalendar.getTime();
        Date startDate = startCalendar.getTime();

        long diff = endDate.getTime() - startDate.getTime();
        int numDays = (int) (diff / (1000 * 60 * 60 * 24));
        int numMonths = numDays / 30;
        int overflowDays = numDays % 30;

        String language = Locale.getDefault().getLanguage();

        if (numMonths > 0) {
            if (language.equals("ca")) {
                return "Fa " + numMonths + " mesos i " + overflowDays + " dies";
            } else if (language.equals("es")) {
                return "Hace " + numMonths + " meses y " + overflowDays + " dias";
            } else {
                return numMonths + " months and " + overflowDays + " days ago";
            }
        } else if (numDays > 0) {
            if (language.equals("ca")) {
                return "Fa " + numDays + " dies";
            } else if (language.equals("es")) {
                return "Hace " + numDays + " dias";
            } else {
                return numDays + " days ago";
            }
        } else {
            if (language.equals("ca")) {
                return "Avui";
            } else if (language.equals("es")) {
                return "Hoy";
            } else {
                return "Today";
            }
        }
    }

    public static void hideKeyboard(Context mContext) {
        InputMethodManager imm = (InputMethodManager) mContext
                .getSystemService(Context.INPUT_METHOD_SERVICE);
        imm.hideSoftInputFromWindow(((Activity) mContext).getWindow()
                .getCurrentFocus().getWindowToken(), 0);
    }
}
