package com.dartmaker.uponevent;

import android.content.Context;
import android.content.Intent;
import android.os.Bundle;
import android.support.v7.app.AppCompatActivity;
import android.support.v7.widget.LinearLayoutManager;
import android.support.v7.widget.RecyclerView;
import android.support.v7.widget.Toolbar;
import android.text.TextUtils;
import android.view.View;
import android.widget.EditText;
import android.widget.ImageView;
import android.widget.TextView;

import com.firebase.ui.database.FirebaseRecyclerAdapter;
import com.google.firebase.auth.FirebaseAuth;
import com.google.firebase.auth.FirebaseUser;
import com.google.firebase.database.DataSnapshot;
import com.google.firebase.database.DatabaseError;
import com.google.firebase.database.DatabaseReference;
import com.google.firebase.database.FirebaseDatabase;
import com.google.firebase.database.ServerValue;
import com.google.firebase.database.ValueEventListener;
import com.squareup.picasso.Picasso;

import java.text.SimpleDateFormat;
import java.util.Date;

public class CommentActivity extends AppCompatActivity {

    private String post_fireBaseKey;
    private String event_fireBaseKey;
    private RecyclerView mUponEventList;
    private DatabaseReference mCommentsDatabase;
    private DatabaseReference mUsersDatabase;
    private FirebaseAuth mAuth;
    private EditText itemComment_postComment_editText;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_comment);

        //INICIALITZEM LES REFERENCIES PRINCIPALS
        mUsersDatabase = FirebaseDatabase.getInstance().getReference().child("Users");
        Intent intent = getIntent();
        mAuth = FirebaseAuth.getInstance();
        Toolbar toolbar = findViewById(R.id.toolbar);
        setSupportActionBar(toolbar);

        //RECUPEREM ELS ID DE PUBLICACIO O D'ESDEVENIMENT
        event_fireBaseKey = intent.getStringExtra("event_fireBaseKey");
        post_fireBaseKey = intent.getStringExtra("post_fireBaseKey");

        //SELECCIONEM EL NODE CORRESPONENT EN EN FUNCIO DE SI ES UN ESDEVENIMENT O UNA PUBLICACIO
        if(!TextUtils.isEmpty(post_fireBaseKey)){
            mCommentsDatabase = FirebaseDatabase.getInstance().getReference().child("Posts").child(post_fireBaseKey).child("comments");
        } else if(!TextUtils.isEmpty(event_fireBaseKey)){
            mCommentsDatabase = FirebaseDatabase.getInstance().getReference().child("Events").child(event_fireBaseKey).child("comments");
        }

        //INICIALITZEM LES VARIABLES
        itemComment_postComment_editText = findViewById(R.id.contentComment_postComment_editText);
        mUponEventList = findViewById(R.id.contentComment_commentList_recycler);
        mUponEventList.setHasFixedSize(true);
        mUponEventList.setLayoutManager(new LinearLayoutManager(this));

    }

    @Override
    protected void onStart() {
        super.onStart();

        //RECUPEREM ELS COMENTARIS DE LA BASE DE DADES AMB EL MODEL COMMENTMODEL I OMPLIM LES VISTES ITEM_COMMENT MITJANÇANT LA CLASSE COMMENTITEMVIEWHOLDER
        final FirebaseRecyclerAdapter<CommentModel, CommentActivity.CommentItemViewHolder> FBRA = new FirebaseRecyclerAdapter<CommentModel, CommentActivity.CommentItemViewHolder>(CommentModel.class, R.layout.item_comment, CommentActivity.CommentItemViewHolder.class, mCommentsDatabase) {
            @Override
            protected void populateViewHolder(final CommentActivity.CommentItemViewHolder viewHolder, CommentModel model, int position) {

                mUsersDatabase.child(model.getUserId()).addValueEventListener(new ValueEventListener() {
                    @Override
                    public void onDataChange(DataSnapshot dataSnapshot) {
                        UserModel user = dataSnapshot.getValue(UserModel.class);
                        viewHolder.setUserName(user.getName());
                        viewHolder.setUserImage(getApplicationContext(), user.getImage());
                    }

                    @Override
                    public void onCancelled(DatabaseError databaseError) {

                    }
                });

                viewHolder.setComment(model.getComment());
                viewHolder.setDatetime(model.getTimestamp());
            }
        };

        mUponEventList.setAdapter(FBRA);
    }

    //CLASSE AUXILIAR PER OMPLIR ELS LAYOUTS ITEM_COMMENT
    public static class CommentItemViewHolder extends RecyclerView.ViewHolder{
        public CommentItemViewHolder(View itemView){
            super(itemView);
            View mView = itemView;
        }

        public void setUserName(String userName){
            TextView itemComment_username_textView = itemView.findViewById(R.id.itemComment_userName_textView);
            itemComment_username_textView.setText(userName);
        }

        public void setComment(String comment){
            TextView itemComment_comment_textView = itemView.findViewById(R.id.itemComment_comment_textView);
            itemComment_comment_textView.setText(comment);
        }

        public void setUserImage(Context ctx, String userImage) {
            ImageView itemComment_userImage_imageView = itemView.findViewById(R.id.itemComment_userImage_imageView);
            Picasso.with(ctx).load(userImage).into(itemComment_userImage_imageView);
        }

        public void setDatetime(long timestamp){
            SimpleDateFormat sfd = new SimpleDateFormat("dd/MM/yyyy HH:mm");
            String dateTimeString = sfd.format(new Date(timestamp)).toString();
            TextView itemComment_datetime_textView = itemView.findViewById(R.id.itemComment_datetime_textView);
            itemComment_datetime_textView.setText(dateTimeString);
        }
    }

    //DESEM LA INFORMACIO
    public void contentComment_postCommentButtonClicked(View view){

        final String comment_text = itemComment_postComment_editText.getText().toString().trim();

        if(!TextUtils.isEmpty(comment_text)){
            FirebaseUser user = mAuth.getCurrentUser();
            DatabaseReference newComment = mCommentsDatabase.push();

            newComment.child("userId").setValue(user.getUid());
            newComment.child("comment").setValue(comment_text);
            newComment.child("timestamp").setValue(ServerValue.TIMESTAMP);

            itemComment_postComment_editText.setText("");
            itemComment_postComment_editText.clearFocus();
            ApplicationUtils.hideKeyboard(CommentActivity.this);

        }
    }

}