package com.dartmaker.uponevent;

import android.app.DatePickerDialog;
import android.content.Context;
import android.content.Intent;
import android.graphics.Bitmap;
import android.graphics.drawable.Drawable;
import android.net.Uri;
import android.support.v7.app.AppCompatActivity;
import android.os.Bundle;
import android.text.TextUtils;
import android.view.View;
import android.widget.DatePicker;
import android.widget.EditText;
import android.widget.ImageButton;
import android.widget.TextView;
import android.widget.Toast;

import com.afollestad.materialdialogs.MaterialDialog;
import com.google.android.gms.common.ConnectionResult;
import com.google.android.gms.common.GooglePlayServicesNotAvailableException;
import com.google.android.gms.common.GooglePlayServicesRepairableException;
import com.google.android.gms.common.GooglePlayServicesUtil;
import com.google.android.gms.tasks.OnSuccessListener;
import com.google.firebase.auth.FirebaseAuth;
import com.google.firebase.database.DataSnapshot;
import com.google.firebase.database.DatabaseError;
import com.google.firebase.database.DatabaseReference;
import com.google.firebase.database.FirebaseDatabase;
import com.google.firebase.database.ValueEventListener;
import com.google.firebase.storage.FirebaseStorage;
import com.google.firebase.storage.StorageReference;
import com.google.firebase.storage.UploadTask;
import com.google.android.gms.location.places.Place;
import com.google.android.gms.location.places.ui.PlacePicker;
import com.squareup.picasso.Picasso;
import com.squareup.picasso.Target;

import java.io.ByteArrayOutputStream;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.Locale;

public class EditEventActivity extends AppCompatActivity {
    private static final int GALLERY_REQUEST = 2;
    private static final int PLACE_PICKER_REQUEST = 3;
    private static final int SELECT_USERS_REQUEST = 8;

    //Camp titol
    private String title;
    private EditText editEvent_title_editText;

    //Camps descripcio
    private String desc;
    private EditText editEvent_description_editText;

    //Camps Imatge
    private String imageName;
    private Bitmap imageBitmap;
    private ImageButton editEvent_image_imageButton;

    //Camps datetime
    private long eventTimestamp;
    private EditText editEvent_datePicker_editText;

    //Camps ubicacio
    private String locationName;
    private double latitude;
    private double longitude;
    private TextView editEvent_locationSelected_textView;

    //CAMP USUARIS
    private TextView editEvent_users_textView;
    private ImageButton editEvent_users_icon;

    //Referencies principals
    private String event_fireBaseKey;
    private StorageReference storageReference;
    private DatabaseReference mDatabase;
    private FirebaseAuth mAuth;
    int status;
    private ApplicationUtils applicationUtils;
    private boolean alreadyStarted;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_edit_event);

        //INICIALITZEM ELS VALORS
        latitude = 0;
        longitude = 0;
        eventTimestamp = 0;
        editEvent_title_editText = findViewById(R.id.editEvent_title_editText);
        editEvent_description_editText = findViewById(R.id.editEvent_description_editText);
        editEvent_locationSelected_textView = findViewById(R.id.editEvent_locationSelected_textView);
        editEvent_datePicker_editText = findViewById(R.id.editEvent_datePicker_editText);
        editEvent_image_imageButton = findViewById(R.id.editEvent_image_imageButton);
        editEvent_users_textView = findViewById(R.id.editEvent_users_textView);
        editEvent_users_icon = findViewById(R.id.editEvent_users_icon);

        //INICIALITZEM LES REFERENCIES PRINCIPALS
        Intent intent = getIntent();
        event_fireBaseKey = intent.getStringExtra("event_fireBaseKey");
        mDatabase = FirebaseDatabase.getInstance().getReference().child("Events").child(event_fireBaseKey);
        alreadyStarted = false;
        storageReference = FirebaseStorage.getInstance().getReference();
        mAuth = FirebaseAuth.getInstance();
        applicationUtils = new ApplicationUtils();

        status = GooglePlayServicesUtil.isGooglePlayServicesAvailable(this);
        if (status != ConnectionResult.SUCCESS) {
            if (GooglePlayServicesUtil.isUserRecoverableError(status)) {
                GooglePlayServicesUtil.getErrorDialog(status, this,
                        100).show();
            }
        }
    }

    @Override
    protected void onStart() {
        super.onStart();

        //COMPROVEM SI ES TRACTA D'UNA PRIMERA ENTRADA A L'ACTIVITY O SI PER CONTRA CAL CARREGAR ELS VALORS JA DESATS
        if (!alreadyStarted) {
            ValueEventListener valueEventListener = new ValueEventListener() {
                @Override
                public void onDataChange(DataSnapshot dataSnapshot) {
                    EventModel event = dataSnapshot.getValue(EventModel.class);

                    //CARREGUEM LES VARIABLES DE LA BASE DE DADES
                    title = event.getTitle();
                    editEvent_title_editText.setText(title);

                    desc = event.getDesc();
                    editEvent_description_editText.setText(desc);

                    if(!dataSnapshot.hasChild("usersInEvent")){
                        editEvent_users_textView.setText(getResources().getString(R.string.Public_event));
                        editEvent_users_icon.setBackground(getResources().getDrawable(R.drawable.ic_lock_open_white_35dp));
                    }

                    Picasso.with(getApplicationContext()).load(event.getImage()).into(new Target() {
                        @Override
                        public void onBitmapLoaded(Bitmap bitmap, Picasso.LoadedFrom from) {
                            imageBitmap = bitmap;
                            editEvent_image_imageButton.setImageBitmap(bitmap);
                            editEvent_image_imageButton.setBackgroundColor(getResources().getColor(android.R.color.transparent));
                        }

                        @Override
                        public void onBitmapFailed(Drawable errorDrawable) {

                        }

                        @Override
                        public void onPrepareLoad(Drawable placeHolderDrawable) {

                        }
                    });

                    locationName = event.getLocationName();
                    editEvent_locationSelected_textView.setText(locationName);

                    eventTimestamp = event.getEventTimestamp();
                    SimpleDateFormat sfd = new SimpleDateFormat("dd/MM/yyyy");
                    String dateTimeString = sfd.format(new Date(eventTimestamp)).toString();
                    editEvent_datePicker_editText.setText(dateTimeString);

                    alreadyStarted = true;
                }

                @Override
                public void onCancelled(DatabaseError databaseError) {

                }
            };

            mDatabase.addValueEventListener(valueEventListener);
        } else {
            //EMPREM LES VARIABLES JA EXISTENTS, NO LES DE BASE DE DADES
            editEvent_title_editText.setText(title);
            editEvent_description_editText.setText(desc);
            editEvent_image_imageButton.setImageBitmap(imageBitmap);
            editEvent_image_imageButton.setBackgroundColor(getResources().getColor(android.R.color.transparent));
            editEvent_locationSelected_textView.setText(locationName);
            SimpleDateFormat sfd = new SimpleDateFormat("dd/MM/yyyy");
            String dateTimeString = sfd.format(new Date(eventTimestamp)).toString();
            editEvent_datePicker_editText.setText(dateTimeString);
        }
    }

    //DESEM ELS VALORS DELS CAMPS DE TEXT EN LES CORRESPONENTS VARIABLES PER QUAN ES TORNI CRIDAR ONSTART
    @Override
    protected void onSaveInstanceState(Bundle outState) {
        super.onSaveInstanceState(outState);
        title = editEvent_title_editText.getText().toString();
        desc = editEvent_description_editText.getText().toString();
    }

    //SELECCIONEM IMATGE
    public void imageButtonClicked(View view) {
        Intent galleryIntent = new Intent();
        galleryIntent.setType("image/*");
        galleryIntent.setAction(Intent.ACTION_GET_CONTENT);
        startActivityForResult(galleryIntent, GALLERY_REQUEST);
    }

    //SELECCIONEM UBICACIO
    public void locationButtonClicked(View view) {
        if (status == ConnectionResult.SUCCESS) {
            PlacePicker.IntentBuilder builder = new PlacePicker.IntentBuilder();
            Context context = this;
            try {
                startActivityForResult(builder.build(this), PLACE_PICKER_REQUEST);
            } catch (GooglePlayServicesRepairableException e) {
                e.printStackTrace();
            } catch (GooglePlayServicesNotAvailableException e) {
                e.printStackTrace();
            }
        }
    }

    public void usersButtonClicked(View view){
        Intent editEvent_userListSelectable_intent = new Intent(EditEventActivity.this, UserListSelectableActivity.class);
        editEvent_userListSelectable_intent.putExtra("event_fireBaseKey", event_fireBaseKey);
        startActivityForResult(editEvent_userListSelectable_intent,SELECT_USERS_REQUEST);
    }

    //SELECCIONEM DATA
    public void datePickerButtonClicked(View view) {
        DatePickerFragment newFragment = DatePickerFragment.newInstance(new DatePickerDialog.OnDateSetListener() {
            @Override

            public void onDateSet(DatePicker datePicker, int year, int month, int day) {
                // +1 JA QUE GENER ES 0
                final String selectedDate = day + " / " + (month + 1) + " / " + year;
                editEvent_datePicker_editText.setText(selectedDate);

                try {
                    String str_date = (month + 1) + "-" + day + "-" + year + " 00:00:00";
                    SimpleDateFormat dateTimeFormater = new SimpleDateFormat("MM-dd-yyyy HH:mm:ss", Locale.getDefault());
                    Date date = dateTimeFormater.parse(str_date);
                    long output = date.getTime() / 1000L;
                    String str = Long.toString(output);
                    eventTimestamp = Long.parseLong(str) * 1000;
                } catch (ParseException e) {
                    e.printStackTrace();
                }
            }
        });
        newFragment.show(this.getFragmentManager(), "datePicker");
    }

    //TRACTEM ELS RETORNS DE SELECCIO DE IMATGE I D'UBICACIO
    @Override
    protected void onActivityResult(int requestCode, int resultCode, Intent data) {
        super.onActivityResult(requestCode, resultCode, data);

        if (requestCode == GALLERY_REQUEST && resultCode == RESULT_OK) {
            imageBitmap = applicationUtils.getCompressedBitmap(data.getData(), getApplicationContext());
            imageName = data.getData().getLastPathSegment();
            editEvent_image_imageButton.setImageBitmap(imageBitmap);
            editEvent_image_imageButton.setBackgroundColor(getResources().getColor(android.R.color.transparent));
        } else if (requestCode == PLACE_PICKER_REQUEST && resultCode == RESULT_OK) {
            Place place = PlacePicker.getPlace(this, data);
            latitude = place.getLatLng().latitude;
            longitude = place.getLatLng().longitude;
            locationName = place.getName().toString();
            String locationSelected = String.format("Ubicació: %s", place.getName());
            editEvent_locationSelected_textView.setText(locationSelected);
        } else if (requestCode == SELECT_USERS_REQUEST && resultCode == RESULT_OK){
            mDatabase.addValueEventListener(new ValueEventListener() {
                @Override
                public void onDataChange(DataSnapshot dataSnapshot) {
                    if(dataSnapshot.hasChild("usersInEvent")){
                        editEvent_users_textView.setText(getResources().getString(R.string.Private_event));
                        editEvent_users_icon.setBackground(getResources().getDrawable(R.drawable.ic_lock_outline_black_35dp));
                    }
                    else {
                        editEvent_users_textView.setText(getResources().getString(R.string.Public_event));
                        editEvent_users_icon.setBackground(getResources().getDrawable(R.drawable.ic_lock_open_white_35dp));
                    }
                }

                @Override
                public void onCancelled(DatabaseError databaseError) {

                }
            });
        }
    }

    //DESEM L'ESDEVENIMENT
    public void submitButtonClicked(View view) {
        final String titleValue = editEvent_title_editText.getText().toString().trim();
        final String descValue = editEvent_description_editText.getText().toString().trim();
        if (!TextUtils.isEmpty(titleValue) && !TextUtils.isEmpty(descValue)) {

            //SI LA IMATGE HA SIGUT CANVIADA, ESPERAREM A LA SEVA CORRECTA CARREGA ABANS DE ACTUALITZAR LES DADES
            if (!TextUtils.isEmpty(imageName)) {

                final MaterialDialog dialog = new MaterialDialog.Builder(view.getContext())
                        .title(getResources().getString(R.string.Uploading_image))
                        .content(getResources().getString(R.string.Please_wait))
                        .progress(true, 100)
                        .show();

                StorageReference filePath = storageReference.child("EventImage").child(imageName);

                ByteArrayOutputStream baos = new ByteArrayOutputStream();
                imageBitmap.compress(Bitmap.CompressFormat.JPEG, 100, baos);
                byte[] data = baos.toByteArray();

                filePath.putBytes(data).addOnSuccessListener(new OnSuccessListener<UploadTask.TaskSnapshot>() {
                    @Override
                    public void onSuccess(UploadTask.TaskSnapshot taskSnapshot) {
                        dialog.dismiss();
                        Uri downloadurl = taskSnapshot.getDownloadUrl();
                        Toast.makeText(EditEventActivity.this, getResources().getString(R.string.Upload_complete), Toast.LENGTH_LONG).show();
                        DatabaseReference editEvent = mDatabase;
                        editEvent.child("title").setValue(titleValue);
                        editEvent.child("desc").setValue(descValue);
                        editEvent.child("image").setValue(downloadurl.toString());
                        if (latitude != 0) {
                            editEvent.child("latitude").setValue(latitude);
                        }

                        if (longitude != 0) {
                            editEvent.child("longitude").setValue(longitude);
                        }

                        if (!TextUtils.isEmpty(locationName)) {
                            editEvent.child("locationName").setValue(locationName);
                        }

                        if (eventTimestamp != 0) {
                            editEvent.child("eventTimestamp").setValue(eventTimestamp);
                        }

                        finish();
                    }
                });
            } else {
                //LA IMATGE NO HA SIGUT CANVIADA, ACTUALITZEM NOMES LES DADES

                DatabaseReference editEvent = mDatabase;
                editEvent.child("title").setValue(titleValue);
                editEvent.child("desc").setValue(descValue);

                if (latitude != 0) {
                    editEvent.child("latitude").setValue(latitude);
                }

                if (longitude != 0) {
                    editEvent.child("longitude").setValue(longitude);
                }

                if (!TextUtils.isEmpty(locationName)) {
                    editEvent.child("locationName").setValue(locationName);
                }

                if (eventTimestamp != 0) {
                    editEvent.child("eventTimestamp").setValue(eventTimestamp);
                }

                finish();
            }
        }

    }
}
