package com.dartmaker.uponevent;

import android.app.DatePickerDialog;
import android.content.Intent;
import android.graphics.Bitmap;
import android.graphics.drawable.Drawable;
import android.net.Uri;
import android.support.v7.app.AppCompatActivity;
import android.os.Bundle;
import android.text.TextUtils;
import android.view.View;
import android.widget.DatePicker;
import android.widget.EditText;
import android.widget.ImageButton;

import com.afollestad.materialdialogs.MaterialDialog;
import com.google.android.gms.tasks.OnSuccessListener;
import com.google.firebase.auth.FirebaseAuth;
import com.google.firebase.auth.UserProfileChangeRequest;
import com.google.firebase.database.DataSnapshot;
import com.google.firebase.database.DatabaseError;
import com.google.firebase.database.DatabaseReference;
import com.google.firebase.database.FirebaseDatabase;
import com.google.firebase.database.ValueEventListener;
import com.google.firebase.storage.FirebaseStorage;
import com.google.firebase.storage.StorageReference;
import com.google.firebase.storage.UploadTask;
import com.squareup.picasso.Picasso;
import com.squareup.picasso.Target;
import com.theartofdev.edmodo.cropper.CropImage;
import com.theartofdev.edmodo.cropper.CropImageView;

import java.io.ByteArrayOutputStream;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.Locale;

public class EditUserActivity extends AppCompatActivity {

    private ImageButton editUser_image_imageButton;
    private Bitmap imageBitmap;
    private String imageName;

    private String userName;
    private EditText editUser_userName_EditText;

    private String name;
    private EditText editUser_name_EditText;

    private String surname;
    private EditText editUser_surname_EditText;

    private String location;
    private EditText editUser_location_EditText;

    private String state;
    private EditText editUser_state_EditText;

    private long birthdayTimestamp;
    private EditText editUser_birthday_editText;

    private static final int GALLERY_RQ = 1;
    private FirebaseAuth mAuth;
    private DatabaseReference mDatabaseUser;
    private StorageReference mStorageref;
    private ApplicationUtils applicationUtils;
    private boolean alreadyStarted;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_edit_user);

        //INICIALITZEM ELS VALORS
        editUser_image_imageButton = findViewById(R.id.editUser_image_imageButton);
        editUser_userName_EditText = findViewById(R.id.editUser_userName_EditText);
        editUser_name_EditText = findViewById(R.id.editUser_name_EditText);
        editUser_surname_EditText = findViewById(R.id.editUser_surname_EditText);
        editUser_location_EditText = findViewById(R.id.editUser_location_EditText);
        editUser_state_EditText = findViewById(R.id.editUser_state_EditText);
        editUser_birthday_editText = findViewById(R.id.editUser_birthday_editText);
        birthdayTimestamp = 0;

        //INICIALITZEM LES REFERENCIES PRINCIPALS
        mAuth = FirebaseAuth.getInstance();
        String user_id = mAuth.getCurrentUser().getUid();
        mDatabaseUser = FirebaseDatabase.getInstance().getReference().child("Users").child(user_id);
        mStorageref = FirebaseStorage.getInstance().getReference().child("profile_image");
        applicationUtils = new ApplicationUtils();
        alreadyStarted = false;
    }

    @Override
    protected void onStart() {
        super.onStart();

        mDatabaseUser.addValueEventListener(new ValueEventListener() {
            @Override
            public void onDataChange(DataSnapshot dataSnapshot) {

                //COMPROVEM SI ES TRACTA D'UNA PRIMERA ENTRADA A L'ACTIVITY O SI PER CONTRA CAL CARREGAR ELS VALORS JA DESATS
                if (!alreadyStarted) {
                    UserModel user = dataSnapshot.getValue(UserModel.class);

                    //CARREGUEM LES VARIABLES DE LA BASE DE DADES
                    if (!user.getImage().equals("default")) {
                        Picasso.with(getApplicationContext()).load(user.getImage()).into(new Target() {
                            @Override
                            public void onBitmapLoaded(Bitmap bitmap, Picasso.LoadedFrom from) {
                                imageBitmap = bitmap;
                                editUser_image_imageButton.setImageBitmap(bitmap);
                                editUser_image_imageButton.setBackgroundColor(getResources().getColor(android.R.color.transparent));
                            }

                            @Override
                            public void onBitmapFailed(Drawable errorDrawable) {

                            }

                            @Override
                            public void onPrepareLoad(Drawable placeHolderDrawable) {

                            }
                        });
                    }

                    if (!TextUtils.isEmpty(user.getUsername())) {
                        userName = user.getUsername();
                        editUser_userName_EditText.setText(userName);
                    }

                    if (!TextUtils.isEmpty(user.getName())) {
                        name = user.getName();
                        editUser_name_EditText.setText(name);
                    }

                    if (!TextUtils.isEmpty(user.getSurname())) {
                        surname = user.getSurname();
                        editUser_surname_EditText.setText(surname);
                    }

                    if (!TextUtils.isEmpty(user.getLocation())) {
                        location = user.getLocation();
                        editUser_location_EditText.setText(location);
                    }

                    if (!TextUtils.isEmpty(user.getState())) {
                        state = user.getState();
                        editUser_state_EditText.setText(state);
                    }

                    if (user.getBirthday() != 0) {
                        birthdayTimestamp = user.getBirthday();
                        SimpleDateFormat sfd = new SimpleDateFormat("dd/MM/yyyy");
                        String dateTimeString = sfd.format(new Date(birthdayTimestamp)).toString();
                        editUser_birthday_editText.setText(dateTimeString);
                    }

                    alreadyStarted = true;
                }
                else {
                    //EMPREM LES VARIABLES JA EXISTENTS, NO LES DE BASE DE DADES
                    editUser_image_imageButton.setImageBitmap(imageBitmap);
                    editUser_image_imageButton.setBackgroundColor(getResources().getColor(android.R.color.transparent));
                    editUser_userName_EditText.setText(userName);
                    editUser_name_EditText.setText(name);
                    editUser_surname_EditText.setText(surname);
                    editUser_location_EditText.setText(location);
                    editUser_state_EditText.setText(state);
                    SimpleDateFormat sfd = new SimpleDateFormat("dd/MM/yyyy");
                    String dateTimeString = sfd.format(new Date(birthdayTimestamp)).toString();
                    editUser_birthday_editText.setText(dateTimeString);
                }
            }

            @Override
            public void onCancelled(DatabaseError databaseError) {

            }
        });
    }

    //DESEM ELS VALORS DELS CAMPS DE TEXT EN LES CORRESPONENTS VARIABLES PER QUAN ES TORNI CRIDAR ONSTART
    @Override
    protected void onSaveInstanceState(Bundle outState) {
        super.onSaveInstanceState(outState);
        userName = editUser_userName_EditText.getText().toString();
        name = editUser_name_EditText.getText().toString();
        surname = editUser_surname_EditText.getText().toString();
        location = editUser_location_EditText.getText().toString();
        state = editUser_state_EditText.getText().toString();
    }

    //SELECCIONEM LA IMATGE
    public void profileImageButtonClicked(View view) {
        Intent galleryIntent = new Intent();
        galleryIntent.setAction(Intent.ACTION_GET_CONTENT);
        galleryIntent.setType("image/*");
        startActivityForResult(galleryIntent, GALLERY_RQ);
    }

    //TRACTAMENT DE LA RESPOSTA DE LA SELECCIO DE IMATGE
    @Override
    protected void onActivityResult(int requestCode, int resultCode, Intent data) {
        super.onActivityResult(requestCode, resultCode, data);

        //RETORN DE LA SELECCIO D'IMATGE, PASSEM A CROPIMAGE
        if (requestCode == GALLERY_RQ && resultCode == RESULT_OK) {
            Uri imageUri = data.getData();
            CropImage.activity(imageUri)
                    .setGuidelines(CropImageView.Guidelines.ON)
                    .setAspectRatio(1, 1)
                    .start(this);
        }

        //RETORN DE CROPIMAGE, DESEM LES DADES EN VARIABLES I CONTINUEM L'EXECUCIO
        if (requestCode == CropImage.CROP_IMAGE_ACTIVITY_REQUEST_CODE) {
            CropImage.ActivityResult result = CropImage.getActivityResult(data);
            if (resultCode == RESULT_OK) {
                imageBitmap = applicationUtils.getCompressedBitmap(result.getUri(), getApplicationContext());
                imageName = result.getUri().getLastPathSegment();
                editUser_image_imageButton.setImageBitmap(imageBitmap);
                editUser_image_imageButton.setBackgroundColor(getResources().getColor(android.R.color.transparent));
            } else if (resultCode == CropImage.CROP_IMAGE_ACTIVITY_RESULT_ERROR_CODE) {
                Exception error = result.getError();
            }
        }
    }

    //SELECCIONEM LA DATA DE NAIXEMENT
    public void datePickerButtonClicked(View view) {
        DatePickerFragment newFragment = DatePickerFragment.newInstance(new DatePickerDialog.OnDateSetListener() {
            @Override

            public void onDateSet(DatePicker datePicker, int year, int month, int day) {
                // +1 JA QUE GENER ES 0
                final String selectedDate = day + " / " + (month + 1) + " / " + year;
                editUser_birthday_editText.setText(selectedDate);

                try {
                    String str_date = (month + 1) + "-" + day + "-" + year + " 00:00:00";
                    SimpleDateFormat dateTimeFormater = new SimpleDateFormat("MM-dd-yyyy HH:mm:ss", Locale.getDefault());
                    Date date = (Date) dateTimeFormater.parse(str_date);
                    long output = date.getTime() / 1000L;
                    String str = Long.toString(output);
                    birthdayTimestamp = Long.parseLong(str) * 1000;
                } catch (ParseException e) {
                    e.printStackTrace();
                }
            }
        });
        newFragment.show(this.getFragmentManager(), "datePicker");
    }

    //DESEM L'EDICIO
    public void doneButtonClicked(View view) {
        final String username = editUser_userName_EditText.getText().toString().trim();
        final String name = editUser_name_EditText.getText().toString();
        final String surname = editUser_surname_EditText.getText().toString();
        final String location = editUser_location_EditText.getText().toString();
        final String state = editUser_state_EditText.getText().toString();

        if (!TextUtils.isEmpty(username)) {

            //SI LA IMATGE HA SIGUT CANVIADA, ESPERAREM A LA SEVA CORRECTA CARREGA ABANS D'ACTUALITZAR LES DADES
            if (!TextUtils.isEmpty(imageName)) {

                final MaterialDialog dialog = new MaterialDialog.Builder(view.getContext())
                        .title(getResources().getString(R.string.Uploading_image))
                        .content(getResources().getString(R.string.Please_wait))
                        .progress(true, 100)
                        .show();

                StorageReference filepath = mStorageref.child(imageName);

                ByteArrayOutputStream baos = new ByteArrayOutputStream();
                imageBitmap.compress(Bitmap.CompressFormat.JPEG, 100, baos);
                byte[] data = baos.toByteArray();

                filepath.putBytes(data).addOnSuccessListener(new OnSuccessListener<UploadTask.TaskSnapshot>() {
                    @Override
                    public void onSuccess(UploadTask.TaskSnapshot taskSnapshot) {
                        dialog.dismiss();
                        String downloadUri = taskSnapshot.getDownloadUrl().toString();
                        mDatabaseUser.child("image").setValue(downloadUri);
                        mDatabaseUser.child("username").setValue(username);
                        mDatabaseUser.child("name").setValue(name);
                        mDatabaseUser.child("surname").setValue(surname);
                        mDatabaseUser.child("birthday").setValue(birthdayTimestamp);
                        mDatabaseUser.child("location").setValue(location);
                        mDatabaseUser.child("state").setValue(state);

                        UserProfileChangeRequest profileUpdates = new UserProfileChangeRequest.Builder().setPhotoUri(taskSnapshot.getDownloadUrl()).build();
                        mAuth.getCurrentUser().updateProfile(profileUpdates);

                        Intent editUser_viewUser = new Intent(EditUserActivity.this, ViewUserActivity.class);
                        editUser_viewUser.addFlags(Intent.FLAG_ACTIVITY_CLEAR_TOP);
                        editUser_viewUser.putExtra("user_fireBaseKey", mAuth.getCurrentUser().getUid());
                        startActivity(editUser_viewUser);
                        finish();
                    }
                });
            } else {
                mDatabaseUser.child("username").setValue(username);
                mDatabaseUser.child("name").setValue(name);
                mDatabaseUser.child("surname").setValue(surname);
                mDatabaseUser.child("birthday").setValue(birthdayTimestamp);
                mDatabaseUser.child("location").setValue(location);
                mDatabaseUser.child("state").setValue(state);
                finish();
            }
        }
    }
}
