package com.dartmaker.uponevent;

import android.content.Intent;
import android.graphics.Bitmap;
import android.graphics.drawable.Drawable;
import android.os.Build;
import android.support.v7.app.AppCompatActivity;
import android.os.Bundle;
import android.text.TextUtils;
import android.util.Log;
import android.view.View;

import com.github.chrisbanes.photoview.PhotoView;
import com.google.firebase.database.DataSnapshot;
import com.google.firebase.database.DatabaseError;
import com.google.firebase.database.DatabaseReference;
import com.google.firebase.database.FirebaseDatabase;
import com.google.firebase.database.ValueEventListener;
import com.squareup.picasso.Picasso;
import com.squareup.picasso.Target;
import static android.R.attr.uiOptions;

public class ImageFullScreenActivity extends AppCompatActivity {


    private PhotoView photoView;
    private String post_fireBaseKey;
    private String event_fireBaseKey;
    private String user_fireBaseKey;
    private DatabaseReference mDatabase;


    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_image_full_screen);


        Intent intent = getIntent();
        post_fireBaseKey = intent.getStringExtra("post_fireBaseKey");
        user_fireBaseKey = intent.getStringExtra("user_fireBaseKey");
        event_fireBaseKey = intent.getStringExtra("event_fireBaseKey");

        if(!TextUtils.isEmpty(post_fireBaseKey)){
            mDatabase = FirebaseDatabase.getInstance().getReference().child("Posts").child(post_fireBaseKey);
        } else if(!TextUtils.isEmpty(user_fireBaseKey)){
            mDatabase = FirebaseDatabase.getInstance().getReference().child("Users").child(user_fireBaseKey);
        } else {
            mDatabase = FirebaseDatabase.getInstance().getReference().child("Events").child(event_fireBaseKey);
        }


        photoView = (PhotoView) findViewById(R.id.photo_view);

    }

    @Override
    protected void onStart() {
        super.onStart();

        mDatabase.addValueEventListener(new ValueEventListener() {
            @Override
            public void onDataChange(DataSnapshot dataSnapshot) {

                if(!TextUtils.isEmpty(post_fireBaseKey)){
                    PostModel post = dataSnapshot.getValue(PostModel.class);

                    Picasso.with(getApplicationContext()).load(post.getImage()).into(new Target() {
                        @Override
                        public void onBitmapLoaded(Bitmap bitmap, Picasso.LoadedFrom from) {
                            photoView.setImageBitmap(bitmap);
                        }

                        @Override
                        public void onBitmapFailed(Drawable errorDrawable) {

                        }

                        @Override
                        public void onPrepareLoad(Drawable placeHolderDrawable) {

                        }
                    });
                } else if(!TextUtils.isEmpty(user_fireBaseKey)){
                    UserModel user = dataSnapshot.getValue(UserModel.class);

                    Picasso.with(getApplicationContext()).load(user.getImage()).into(new Target() {
                        @Override
                        public void onBitmapLoaded(Bitmap bitmap, Picasso.LoadedFrom from) {
                            photoView.setImageBitmap(bitmap);
                        }

                        @Override
                        public void onBitmapFailed(Drawable errorDrawable) {

                        }

                        @Override
                        public void onPrepareLoad(Drawable placeHolderDrawable) {

                        }
                    });
                } else {
                    EventModel event = dataSnapshot.getValue(EventModel.class);

                    Picasso.with(getApplicationContext()).load(event.getImage()).into(new Target() {
                        @Override
                        public void onBitmapLoaded(Bitmap bitmap, Picasso.LoadedFrom from) {
                            photoView.setImageBitmap(bitmap);
                        }

                        @Override
                        public void onBitmapFailed(Drawable errorDrawable) {

                        }

                        @Override
                        public void onPrepareLoad(Drawable placeHolderDrawable) {

                        }
                    });
                }
            }

            @Override
            public void onCancelled(DatabaseError databaseError) {

            }
        });
        fullScreen();
    }

    public void fullScreen() {
        int uiOptions = getWindow().getDecorView().getSystemUiVisibility();
        int newUiOptions = uiOptions;
        boolean isImmersiveModeEnabled = isImmersiveModeEnabled();

        if (Build.VERSION.SDK_INT >= 14) {
            newUiOptions ^= View.SYSTEM_UI_FLAG_HIDE_NAVIGATION;
        }

        if (Build.VERSION.SDK_INT >= 16) {
            newUiOptions ^= View.SYSTEM_UI_FLAG_FULLSCREEN;
        }

        if (Build.VERSION.SDK_INT >= 18) {
            newUiOptions ^= View.SYSTEM_UI_FLAG_IMMERSIVE_STICKY;
        }

        getWindow().getDecorView().setSystemUiVisibility(newUiOptions);
    }

    private boolean isImmersiveModeEnabled() {
        return ((uiOptions | View.SYSTEM_UI_FLAG_IMMERSIVE_STICKY) == uiOptions);
    }
}
