package com.dartmaker.uponevent;

import android.content.Context;
import android.content.Intent;
import android.os.Bundle;
import android.support.annotation.NonNull;
import android.support.constraint.solver.widgets.Snapshot;
import android.support.design.widget.BottomNavigationView;
import android.support.v7.app.AppCompatActivity;
import android.support.v7.widget.LinearLayoutManager;
import android.support.v7.widget.RecyclerView;
import android.support.v7.widget.Toolbar;
import android.util.Log;
import android.view.LayoutInflater;
import android.view.View;
import android.view.MenuItem;
import android.view.ViewGroup;
import android.widget.ImageView;
import android.widget.TextView;

import com.firebase.ui.database.FirebaseRecyclerAdapter;
import com.google.firebase.auth.FirebaseAuth;
import com.google.firebase.database.DataSnapshot;
import com.google.firebase.database.DatabaseError;
import com.google.firebase.database.DatabaseReference;
import com.google.firebase.database.FirebaseDatabase;
import com.google.firebase.database.ValueEventListener;
import com.squareup.picasso.Picasso;

import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Date;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.ListIterator;
import java.util.Locale;

public class MainActivity extends AppCompatActivity {

    private RecyclerView mUponEventList;
    private DatabaseReference mDatabase;
    private FirebaseAuth mAuth;
    private FirebaseAuth.AuthStateListener mAuthListener;
    private BottomNavigationView bottomNavigation;
    private int position;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_main);

        //INICIALITZEM LES VARIABLES PRINCIPALS
        mUponEventList = findViewById(R.id.uponEvent_list);
        mUponEventList.setHasFixedSize(true);
        mUponEventList.setLayoutManager(new LinearLayoutManager(this));
        bottomNavigation = findViewById(R.id.bottom_navigation);
        position = 0;

        //INICIALITZEM LES REFERENCIES PRINCIPALS
        Toolbar toolbar = findViewById(R.id.toolbar);
        setSupportActionBar(toolbar);
        mDatabase = FirebaseDatabase.getInstance().getReference().child("Events");

        mAuth = FirebaseAuth.getInstance();
        mAuthListener = new FirebaseAuth.AuthStateListener() {
            @Override
            public void onAuthStateChanged(@NonNull FirebaseAuth firebaseAuth) {
                if(firebaseAuth.getCurrentUser() == null){
                    Intent loginIntent = new Intent(MainActivity.this, LoginActivity.class);
                    loginIntent.addFlags(Intent.FLAG_ACTIVITY_CLEAR_TASK);
                    startActivity(loginIntent);
                }
            }
        };
    }

    @Override
    protected void onStart() {
        super.onStart();

        mAuth.addAuthStateListener(mAuthListener);

        //FUNCIO PER OMPLIR EL RECYCLER AMB ELS ESDEVENIMENTS
        readEvents(new GetEventsCallback() {
            @Override
            public void onCallback(List<HashMap<String,EventModel>> events) {
                EventsAdapter adapter = new EventsAdapter(events);
                mUponEventList.setAdapter(adapter);
                mUponEventList.scrollToPosition(position);
            }
        });

        //CREAM EL MENU HORITZONTAL DE NAVEGACIO PER LA PANTALLA MAIN
        bottomNavigation.setSelectedItemId(R.id.menuMain_home_item);
        bottomNavigation.setOnNavigationItemSelectedListener(new BottomNavigationView.OnNavigationItemSelectedListener() {
            @Override
            public boolean onNavigationItemSelected(@NonNull MenuItem item) {
                int id = item.getItemId();

                if(id == R.id.menuMain_home_item){
                    return true;
                }

                else if(id == R.id.addIcon){
                    Intent intent = new Intent(MainActivity.this, NewEventActivity.class);
                    startActivity(intent);
                    finish();
                }

                else if(id == R.id.menuMain_profile_item){
                    Intent main_viewUser_intent = new Intent(MainActivity.this, ViewUserActivity.class);
                    main_viewUser_intent.putExtra("user_fireBaseKey", mAuth.getCurrentUser().getUid());
                    startActivity(main_viewUser_intent);
                    finish();
                }

                else if(id == R.id.menuMain_search_item){
                    Intent main_search_intent = new Intent(MainActivity.this, SearchActivity.class);
                    startActivity(main_search_intent);
                    finish();
                }

                return true;
            }
        });
    }

    //ADAPTER NECESSARI PER EL VIEWHOLDER
    class EventsAdapter extends RecyclerView.Adapter {

        private List<HashMap<String,EventModel>> models = new ArrayList<>();

        public EventsAdapter(final List<HashMap<String,EventModel>> viewModels) {
            if (viewModels != null) {
                this.models.addAll(viewModels);
            }
        }
        @Override
        public RecyclerView.ViewHolder onCreateViewHolder(final ViewGroup parent, final int viewType) {
            final View view = LayoutInflater.from(parent.getContext()).inflate(viewType, parent, false);
            return new EventItemViewHolder(view);
        }
        @Override
        public void onBindViewHolder(final RecyclerView.ViewHolder holder, final int position) {
            ((EventItemViewHolder) holder).bindData(models.get(position));
        }

        @Override
        public int getItemCount() {
            return models.size();
        }


        @Override
        public int getItemViewType(final int position) {
            return R.layout.item_event;
        }
    }

    //CLASSE VIEWHOLDER PER EMPLENAR ELS LAYOUTS ITEM_EVENT
    public class EventItemViewHolder extends RecyclerView.ViewHolder{

        TextView event_title = itemView.findViewById(R.id.itemEvent_title);
        TextView event_locationName = itemView.findViewById(R.id.itemEvent_locationName);
        TextView event_date = itemView.findViewById(R.id.itemEvent_date);
        ImageView event_image = itemView.findViewById(R.id.itemEvent_image);

        public EventItemViewHolder(View itemView){
            super(itemView);
            View mView = itemView;
        }

        public void bindData(final HashMap<String,EventModel> event) {
            event_title.setText(event.values().iterator().next().getTitle());
            event_locationName.setText(event.values().iterator().next().getLocationName());

            SimpleDateFormat sfd = new SimpleDateFormat("dd/MM/yyyy", Locale.FRANCE);
            String dateTimeString = sfd.format(new Date(event.values().iterator().next().getEventTimestamp()));
            event_date.setText(dateTimeString);

            Picasso.with(getApplicationContext()).load(event.values().iterator().next().getImage()).into(event_image);

            itemView.setOnClickListener(new View.OnClickListener() {
                @Override
                public void onClick(View v) {
                    position = getAdapterPosition();
                    Intent main_viewEvent_Intent = new Intent(MainActivity.this, ViewEventActivity.class);
                    main_viewEvent_Intent.putExtra("event_fireBaseKey", event.keySet().iterator().next());
                    startActivity(main_viewEvent_Intent);
                }
            });
        }
    }

    //CALLBACK PER PODER EMPLENAR ELS ESDEVENIMENTS UN COP CARREGATS
    public interface GetEventsCallback {
        void onCallback(List<HashMap<String,EventModel>> events);
    }

    //FUNCIO QUE RECORRE ELS ESDEVENIMENTS I LES RETORNA AL CALLBACK
    public void readEvents(final GetEventsCallback getEventsCallback) {
        mDatabase.addListenerForSingleValueEvent(new ValueEventListener() {

            @Override
            public void onDataChange(DataSnapshot dataSnapshot) {
                List<HashMap<String,EventModel>> events = new ArrayList<>();
                for (DataSnapshot snapshot : dataSnapshot.getChildren()) {
                    String eventKey = snapshot.getKey();
                    EventModel event = snapshot.getValue(EventModel.class);
                    HashMap<String, EventModel> hashMap = new HashMap<>();
                    hashMap.put(eventKey, event);
                    events.add(hashMap);
                    if (snapshot.hasChild("usersInEvent")) {
                        if (!snapshot.child("usersInEvent").hasChild(mAuth.getCurrentUser().getUid()) && (!snapshot.child("userId").getValue().toString().equals(mAuth.getCurrentUser().getUid()))) {
                            events.remove(hashMap);
                        }
                    }
                }
                getEventsCallback.onCallback(events);

            }

            @Override
            public void onCancelled(DatabaseError databaseError) {

            }
        });
    }
}
