package com.dartmaker.uponevent;

import android.app.DatePickerDialog;
import android.content.Context;
import android.content.Intent;
import android.graphics.Bitmap;
import android.net.Uri;
import android.support.annotation.NonNull;
import android.support.design.widget.BottomNavigationView;
import android.support.v7.app.AppCompatActivity;
import android.os.Bundle;
import android.text.TextUtils;
import android.view.MenuItem;
import android.view.View;
import android.widget.DatePicker;
import android.widget.EditText;
import android.widget.ImageButton;
import android.widget.TextView;
import android.widget.Toast;

import com.afollestad.materialdialogs.MaterialDialog;
import com.google.android.gms.common.ConnectionResult;
import com.google.android.gms.common.GooglePlayServicesNotAvailableException;
import com.google.android.gms.common.GooglePlayServicesRepairableException;
import com.google.android.gms.common.GooglePlayServicesUtil;
import com.google.android.gms.tasks.OnSuccessListener;
import com.google.firebase.auth.FirebaseAuth;
import com.google.firebase.auth.FirebaseUser;
import com.google.firebase.database.DataSnapshot;
import com.google.firebase.database.DatabaseError;
import com.google.firebase.database.DatabaseReference;
import com.google.firebase.database.FirebaseDatabase;
import com.google.firebase.database.ServerValue;
import com.google.firebase.database.ValueEventListener;
import com.google.firebase.storage.FirebaseStorage;
import com.google.firebase.storage.OnProgressListener;
import com.google.firebase.storage.StorageReference;
import com.google.firebase.storage.UploadTask;
import com.google.android.gms.location.places.Place;
import com.google.android.gms.location.places.ui.PlacePicker;

import java.io.ByteArrayOutputStream;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.Locale;

public class NewEventActivity extends AppCompatActivity {
    private static final int GALLERY_REQUEST = 2;
    private static final int PLACE_PICKER_REQUEST = 3;
    private static final int SELECT_USERS_REQUEST = 8;

    private Bitmap imageBitmap;
    private String imageName;
    private ImageButton newEvent_image_imageButton;

    private EditText newEvent_title_editText;
    private EditText newEvent_description_editText;

    private String locationName;
    private double latitude;
    private double longitude;
    private TextView newEvent_locationSelected_textView;

    //CAMPs USUARIS
    private TextView newEvent_users_textView;
    private ArrayList<String> userIds;
    private ImageButton newEvent_users_icon;

    private long eventTimestamp;
    private EditText newEvent_datePicker_editText;

    //REFERENCIES PRINCIPALS
    private ApplicationUtils applicationUtils;
    private StorageReference storageReference;
    private DatabaseReference databaseReference;
    private FirebaseAuth mAuth;
    private BottomNavigationView bottomNavigation;
    int status;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_new_event);

        //INICIALITZEM ELS VALORS
        newEvent_image_imageButton = findViewById(R.id.newEvent_image_imageButton);
        newEvent_title_editText = findViewById(R.id.newEvent_title_editText);
        newEvent_description_editText = findViewById(R.id.newEvent_description_editText);

        latitude = 0;
        longitude = 0;
        newEvent_locationSelected_textView = findViewById(R.id.newEvent_locationSelected_textView);

        newEvent_users_textView = findViewById(R.id.newEvent_users_textView);
        userIds = new ArrayList<>();
        newEvent_users_icon = findViewById(R.id.newEvent_users_icon);

        eventTimestamp = 0;
        newEvent_datePicker_editText = findViewById(R.id.newEvent_datePicker_editText);

        //INICIALITZEM LES REFERENCIES PRINCPALS
        applicationUtils = new ApplicationUtils();
        storageReference = FirebaseStorage.getInstance().getReference();
        databaseReference = FirebaseDatabase.getInstance().getReference().child("Events");
        mAuth = FirebaseAuth.getInstance();
        bottomNavigation = findViewById(R.id.newEvent_bottom_navigation);

        status = GooglePlayServicesUtil.isGooglePlayServicesAvailable(this);
        if (status != ConnectionResult.SUCCESS) {
            if (GooglePlayServicesUtil.isUserRecoverableError(status)) {
                GooglePlayServicesUtil.getErrorDialog(status, this,
                        100).show();
            }
        }
    }

    @Override
    public void onStart() {
        super.onStart();

        //INICIALITZEM EL MENU DE NAVEGACIO I AFEGIM LISTENERS
        bottomNavigation.setSelectedItemId(R.id.addIcon);
        bottomNavigation.setOnNavigationItemSelectedListener(new BottomNavigationView.OnNavigationItemSelectedListener() {
            @Override
            public boolean onNavigationItemSelected(@NonNull MenuItem item) {
                int id = item.getItemId();

                if (id == R.id.menuMain_home_item) {
                    Intent intent = new Intent(NewEventActivity.this, MainActivity.class);
                    startActivity(intent);
                    finish();
                } else if (id == R.id.addIcon) {
                    return true;
                } else if (id == R.id.menuMain_profile_item) {
                    Intent main_viewUser_intent = new Intent(NewEventActivity.this, ViewUserActivity.class);
                    main_viewUser_intent.putExtra("user_fireBaseKey", mAuth.getCurrentUser().getUid());
                    startActivity(main_viewUser_intent);
                    finish();
                } else if (id == R.id.menuMain_search_item) {
                    Intent main_search_intent = new Intent(NewEventActivity.this, SearchActivity.class);
                    startActivity(main_search_intent);
                    finish();
                }
                return true;
            }
        });
    }

    //SELECCIONEM IMATGE
    public void imageButtonClicked(View imageView) {
        Intent galleryIntent = new Intent();
        galleryIntent.setType("image/*");
        galleryIntent.setAction(Intent.ACTION_GET_CONTENT);
        startActivityForResult(galleryIntent, GALLERY_REQUEST);
    }

    //SELECCIONEM UBICACIO
    public void locationButtonClicked(View view) {
        if (status == ConnectionResult.SUCCESS) {
            PlacePicker.IntentBuilder builder = new PlacePicker.IntentBuilder();
            Context context = this;
            try {
                startActivityForResult(builder.build(this), PLACE_PICKER_REQUEST);
            } catch (GooglePlayServicesRepairableException e) {
                e.printStackTrace();
            } catch (GooglePlayServicesNotAvailableException e) {
                e.printStackTrace();
            }
        }
    }

    //SELECCIONEM DATA
    public void datePickerButtonClicked(View view) {
        DatePickerFragment newFragment = DatePickerFragment.newInstance(new DatePickerDialog.OnDateSetListener() {
            @Override

            public void onDateSet(DatePicker datePicker, int year, int month, int day) {
                // +1 because january is zero
                final String selectedDate = day + " / " + (month + 1) + " / " + year;
                newEvent_datePicker_editText.setText(selectedDate);

                try {
                    String str_date = month + "-" + day + "-" + year + " 00:00:00";
                    SimpleDateFormat dateTimeFormater = new SimpleDateFormat("MM-dd-yyyy HH:mm:ss", Locale.getDefault());
                    Date date = (Date) dateTimeFormater.parse(str_date);
                    long output = date.getTime() / 1000L;
                    String str = Long.toString(output);
                    eventTimestamp = Long.parseLong(str) * 1000;
                } catch (ParseException e) {
                    e.printStackTrace();
                }
            }
        });
        newFragment.show(this.getFragmentManager(), "datePicker");
    }

    //TRACTEM ELS RETORNS DE SELECCIO D'IMATGE I D'UBICACIO
    @Override
    protected void onActivityResult(int requestCode, int resultCode, Intent data) {
        super.onActivityResult(requestCode, resultCode, data);

        if (requestCode == GALLERY_REQUEST && resultCode == RESULT_OK) {
            imageBitmap = applicationUtils.getCompressedBitmap(data.getData(), getApplicationContext());
            imageName = data.getData().getLastPathSegment();
            newEvent_image_imageButton.setImageBitmap(imageBitmap);
            newEvent_image_imageButton.setBackgroundColor(getResources().getColor(android.R.color.transparent));
        } else if (requestCode == PLACE_PICKER_REQUEST && resultCode == RESULT_OK) {
            Place place = PlacePicker.getPlace(this, data);
            latitude = place.getLatLng().latitude;
            longitude = place.getLatLng().longitude;
            locationName = place.getName().toString();
            String locationSelected = String.format(getResources().getString(R.string.Location) + ": %s", place.getName());
            newEvent_locationSelected_textView.setText(locationSelected);
        } else if (requestCode == SELECT_USERS_REQUEST && resultCode == RESULT_OK) {
            userIds = data.getStringArrayListExtra("userIds");
            if (userIds.isEmpty()) {
                newEvent_users_textView.setText(getResources().getString(R.string.Public_event));
                newEvent_users_icon.setBackground(getResources().getDrawable(R.drawable.ic_lock_open_white_35dp));
            } else {
                newEvent_users_textView.setText(getResources().getString(R.string.Private_event));
                newEvent_users_icon.setBackground(getResources().getDrawable(R.drawable.ic_lock_outline_black_35dp));
            }
        }
    }

    public void usersButtonClicked(View view) {
        Intent newEvent_userListSelectable_intent = new Intent(NewEventActivity.this, UserListSelectableActivity.class);
        newEvent_userListSelectable_intent.putStringArrayListExtra("userIds", userIds);
        startActivityForResult(newEvent_userListSelectable_intent, SELECT_USERS_REQUEST);
    }

    //DESEM L'ESDEVENIMENT
    public void submitButtonClicked(View view) {
        final String titleValue = newEvent_title_editText.getText().toString().trim();
        final String descValue = newEvent_description_editText.getText().toString().trim();
        final FirebaseUser user = mAuth.getCurrentUser();
        if (!TextUtils.isEmpty(titleValue) && !TextUtils.isEmpty(descValue) && latitude != 0 && longitude != 0 && eventTimestamp != 0 && !TextUtils.isEmpty(imageName)) {

            final MaterialDialog dialog = new MaterialDialog.Builder(view.getContext())
                    .title(getResources().getString(R.string.Uploading_image))
                    .content(getResources().getString(R.string.Please_wait))
                    .progress(true, 100)      // LOADER SENSE PROGRESS
                    .show();

            StorageReference filePath = storageReference.child("EventImage").child(imageName);

            ByteArrayOutputStream baos = new ByteArrayOutputStream();
            imageBitmap.compress(Bitmap.CompressFormat.JPEG, 100, baos);
            byte[] data = baos.toByteArray();

            UploadTask uploadTask = filePath.putBytes(data);

            uploadTask.addOnSuccessListener(new OnSuccessListener<UploadTask.TaskSnapshot>() {
                @Override
                public void onSuccess(UploadTask.TaskSnapshot taskSnapshot) {
                    dialog.dismiss();
                    Uri downloadurl = taskSnapshot.getDownloadUrl();
                    Toast.makeText(NewEventActivity.this, getResources().getString(R.string.Upload_complete), Toast.LENGTH_LONG).show();
                    DatabaseReference newEvent = databaseReference.push();
                    newEvent.child("title").setValue(titleValue);
                    newEvent.child("desc").setValue(descValue);
                    newEvent.child("image").setValue(downloadurl.toString());
                    newEvent.child("userId").setValue(user.getUid());
                    newEvent.child("timestamp").setValue(ServerValue.TIMESTAMP);
                    newEvent.child("latitude").setValue(latitude);
                    newEvent.child("longitude").setValue(longitude);
                    newEvent.child("locationName").setValue(locationName);
                    newEvent.child("eventTimestamp").setValue(eventTimestamp);

                    if (!userIds.isEmpty()) {
                        for (String userId : userIds) {
                            newEvent.child("usersInEvent").child(userId).setValue(true);
                        }
                    }

                    Intent newEvent_viewEvent_intent = new Intent(NewEventActivity.this, ViewEventActivity.class);
                    newEvent_viewEvent_intent.putExtra("event_fireBaseKey", newEvent.getKey());
                    startActivity(newEvent_viewEvent_intent);
                    finish();
                }
            });
        }
    }
}
