package com.dartmaker.uponevent;

import android.content.Intent;
import android.graphics.Bitmap;
import android.net.Uri;
import android.support.v7.app.AppCompatActivity;
import android.os.Bundle;
import android.text.TextUtils;
import android.view.View;
import android.widget.EditText;
import android.widget.ImageButton;
import android.widget.Toast;

import com.afollestad.materialdialogs.MaterialDialog;
import com.google.android.gms.tasks.OnSuccessListener;
import com.google.firebase.auth.FirebaseAuth;
import com.google.firebase.auth.FirebaseUser;
import com.google.firebase.database.DatabaseReference;
import com.google.firebase.database.FirebaseDatabase;
import com.google.firebase.database.ServerValue;
import com.google.firebase.storage.FirebaseStorage;
import com.google.firebase.storage.StorageReference;
import com.google.firebase.storage.UploadTask;

import java.io.ByteArrayOutputStream;

public class NewPostActivity extends AppCompatActivity {

    private static final int GALLERY_REQUEST = 2;
    private Bitmap imageBitmap;
    private String imageName;
    private ImageButton imageButton;
    private EditText editText;
    private StorageReference storageReference;
    private String event_fireBaseKey;
    private DatabaseReference mDatabase;
    private DatabaseReference mDatabasePosts;
    private DatabaseReference mDatabaseUsers;
    private FirebaseAuth mAuth;
    private ApplicationUtils applicationUtils;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_new_post);

        //INICIALITZEM ELS VALORS
        editText = findViewById(R.id.newPost_text_editText);
        imageButton = findViewById(R.id.newPost_image_imageButton);

        //INICIALITZEM LES REFERENCIES PRINCIPALS
        storageReference = FirebaseStorage.getInstance().getReference();
        Intent intent = getIntent();
        event_fireBaseKey = intent.getStringExtra("event_fireBaseKey");
        mDatabase = FirebaseDatabase.getInstance().getReference().child("Events").child(event_fireBaseKey).child("posts");
        mDatabasePosts = FirebaseDatabase.getInstance().getReference().child("Posts");
        mDatabaseUsers = FirebaseDatabase.getInstance().getReference().child("Users");
        mAuth = FirebaseAuth.getInstance();
        applicationUtils = new ApplicationUtils();
    }

    //SELECCIONEM LA IMATGE
    public void imageButtonClicked(View view){
        Intent galleryIntent = new Intent();
        galleryIntent.setType("image/*");
        galleryIntent.setAction(Intent.ACTION_GET_CONTENT);
        startActivityForResult(galleryIntent,GALLERY_REQUEST);
    }

    //TRACTAMENT DEL RETORN DE LA SELECCIO D'IMATGE
    @Override
    protected void onActivityResult(int requestCode, int resultCode, Intent data) {
        super.onActivityResult(requestCode, resultCode, data);
        if(requestCode == GALLERY_REQUEST && resultCode == RESULT_OK){
            imageBitmap = applicationUtils.getCompressedBitmap(data.getData(), getApplicationContext());
            imageName = data.getData().getLastPathSegment();
            imageButton.setImageBitmap(imageBitmap);
            imageButton.setBackgroundColor(getResources().getColor(android.R.color.transparent));
        }
    }

    //DESEM ELS CANVIS DE LA NOVA PUBLICACIO
    public void submitButtonClicked(View view){
        final String textValue = editText.getText().toString().trim();
        final FirebaseUser user = mAuth.getCurrentUser();
        if(!TextUtils.isEmpty(textValue)){

            final MaterialDialog dialog = new MaterialDialog.Builder(view.getContext())
                    .title(getResources().getString(R.string.Uploading_image))
                    .content(getResources().getString(R.string.Please_wait))
                    .progress(true, 100)      // LOADER SENSE PROGRESS
                    //.progress(false, 100, showMinMax)         // DESCOMENTAR PER MOSTRAR EL PROGRES
                    .show();

            StorageReference filePath = storageReference.child("PostImage").child(imageName);

            ByteArrayOutputStream baos = new ByteArrayOutputStream();
            imageBitmap.compress(Bitmap.CompressFormat.JPEG, 100, baos);
            byte[] data = baos.toByteArray();

            filePath.putBytes(data).addOnSuccessListener(new OnSuccessListener<UploadTask.TaskSnapshot>() {
                @Override
                public void onSuccess(UploadTask.TaskSnapshot taskSnapshot) {

                    dialog.dismiss();

                    Uri downloadurl = taskSnapshot.getDownloadUrl();
                    Toast.makeText(NewPostActivity.this,getResources().getString(R.string.Upload_complete), Toast.LENGTH_LONG).show();

                    DatabaseReference newPost = mDatabasePosts.push();
                    newPost.child("text").setValue(textValue);
                    newPost.child("image").setValue(downloadurl.toString());
                    newPost.child("userId").setValue(user.getUid());
                    newPost.child("timestamp").setValue(ServerValue.TIMESTAMP);
                    newPost.child("eventId").setValue(event_fireBaseKey);

                    DatabaseReference newUserPost = mDatabaseUsers.child(user.getUid()).child("posts").child(newPost.getKey());
                    newUserPost.setValue(true);

                    DatabaseReference newEventPost = mDatabase.child(newPost.getKey());
                    newEventPost.setValue(true);

                    Intent post_viewPost_intent = new Intent(NewPostActivity.this, ViewPostActivity.class);
                    post_viewPost_intent.putExtra("post_fireBaseKey", newPost.getKey());
                    startActivity(post_viewPost_intent);
                    finish();
                }
            });
        }
    }
}

