package com.dartmaker.uponevent;

import android.content.Intent;
import android.support.annotation.NonNull;
import android.support.v7.app.AppCompatActivity;
import android.os.Bundle;
import android.text.TextUtils;
import android.view.View;
import android.widget.EditText;
import android.widget.Toast;

import com.google.android.gms.tasks.OnCompleteListener;
import com.google.android.gms.tasks.Task;
import com.google.firebase.auth.AuthResult;
import com.google.firebase.auth.FirebaseAuth;
import com.google.firebase.auth.FirebaseUser;
import com.google.firebase.auth.UserProfileChangeRequest;
import com.google.firebase.database.DatabaseReference;
import com.google.firebase.database.FirebaseDatabase;

public class RegisterActivity extends AppCompatActivity {

    private EditText nameField;
    private EditText emailField;
    private EditText passField;
    private FirebaseAuth mAuth;
    private DatabaseReference mDatabase;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_register);

        //INICIALITZEM ELS VALORS
        nameField = findViewById(R.id.nameField);
        emailField = findViewById(R.id.emailField);
        passField = findViewById(R.id.passField);

        //INICIALITZEM LES REFERENCIES PRINCIPALS
        mAuth = FirebaseAuth.getInstance();
        mDatabase = FirebaseDatabase.getInstance().getReference().child("Users");
    }

    //DESEM EL REGISTRE DEL NOU USUARI
    public void registerButtonClicked(View view){
        final String name = nameField.getText().toString().trim();
        final String email = emailField.getText().toString().trim();
        final String pass = passField.getText().toString().trim();

        //VALIDEM QUE NO HI HAGI CAMPS BUITS
        if(!TextUtils.isEmpty(name) && !TextUtils.isEmpty(email) && !TextUtils.isEmpty(pass)){

            String Email = email;
            String Password = pass;
            mAuth.createUserWithEmailAndPassword(Email, Password)
                    .addOnCompleteListener(RegisterActivity.this, new OnCompleteListener<AuthResult>() {
                        @Override
                        public void onComplete(@NonNull Task<AuthResult> task) {
                            if (task.isSuccessful()) {
                                //REGISTRE CORRECTE, HO MOSTREM PER TOAST
                                Toast.makeText(RegisterActivity.this, getResources().getString(R.string.Authentication_success), Toast.LENGTH_SHORT).show();

                                //DESEM ELS CANVIS A FIREBASEAUTH
                                FirebaseUser user = mAuth.getCurrentUser();
                                UserProfileChangeRequest profileUpdates = new UserProfileChangeRequest.Builder().setDisplayName(name).build();
                                user.updateProfile(profileUpdates);

                                //DESEM ELS CANVIS A LA BASE DE DADES
                                String user_id = user.getUid();
                                DatabaseReference current_user_db = mDatabase.child(user_id);
                                current_user_db.child("name").setValue(name);
                                current_user_db.child("email").setValue(email);
                                current_user_db.child("image").setValue("default");

                                //SALTEM A LA ACTIVITY DE EDICIO D'USUARI PER SI VOL CARREGAR IMATGE DE PERFIL I AFEGIR NOM I LLINATGES
                                Intent register_editUser_intent = new Intent(RegisterActivity.this, EditUserActivity.class);
                                register_editUser_intent.addFlags(Intent.FLAG_ACTIVITY_CLEAR_TOP);
                                startActivity(register_editUser_intent);
                                finish();
                            } else {
                                //REGISTRE FALLIT, HO MOSTREM PER TOAST
                                Toast.makeText(RegisterActivity.this, getResources().getString(R.string.Authentication_failed),  Toast.LENGTH_SHORT).show();
                            }
                        }
                    });
        }
    }
}
