package com.dartmaker.uponevent;

import android.content.Context;
import android.content.Intent;
import android.net.Uri;
import android.os.Bundle;
import android.support.v4.app.Fragment;
import android.support.v7.widget.LinearLayoutManager;
import android.support.v7.widget.RecyclerView;
import android.text.TextUtils;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.EditText;
import android.widget.ImageButton;
import android.widget.ImageView;
import android.widget.TextView;

import com.firebase.ui.database.FirebaseRecyclerAdapter;
import com.google.firebase.auth.FirebaseAuth;
import com.google.firebase.database.DataSnapshot;
import com.google.firebase.database.DatabaseError;
import com.google.firebase.database.DatabaseReference;
import com.google.firebase.database.FirebaseDatabase;
import com.google.firebase.database.Query;
import com.google.firebase.database.ValueEventListener;
import com.squareup.picasso.Picasso;

import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Locale;

public class SearchEventFragment extends Fragment {

    protected EditText searchEvent_search_editText;
    protected ImageButton searchEvent_searchButton_imageButton;
    protected RecyclerView searchEvent_eventList_recycler;
    protected DatabaseReference mDatabase_events;
    private FirebaseAuth mAuth;
    protected Query mQuery;

    public SearchEventFragment() {
        // Required empty public constructor
    }

    @Override
    public void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
    }

    @Override
    public View onCreateView(LayoutInflater inflater, ViewGroup container,
                             Bundle savedInstanceState) {

        View view = inflater.inflate(R.layout.fragment_search_event, container, false);

        //INICIALITZEM LES VARIABLES
        searchEvent_search_editText = view.findViewById(R.id.searchEvent_search_editText);
        searchEvent_searchButton_imageButton = view.findViewById(R.id.searchEvent_searchButton_imageButton);
        searchEvent_eventList_recycler = view.findViewById(R.id.searchEvent_eventList_recycler);
        searchEvent_eventList_recycler.setLayoutManager(new LinearLayoutManager(getContext()));
        searchEvent_eventList_recycler.setHasFixedSize(true);

        //INICIALITZEM LES REFERENCIES PRINCIPALS
        mDatabase_events = FirebaseDatabase.getInstance().getReference().child("Events");
        mAuth = FirebaseAuth.getInstance();

        searchEvent_searchButton_imageButton.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                searchEvent_searchButtonClicked();
                ApplicationUtils.hideKeyboard(getContext());
                searchEvent_search_editText.clearFocus();
            }
        });

        return view;
    }

    //FILTREM ELS RESULTATS
    public void searchEvent_searchButtonClicked(){

        if(!TextUtils.isEmpty(searchEvent_search_editText.getText().toString())){
            String searchField = searchEvent_search_editText.getText().toString();

            //QUERY DE FILTRATGE
            mQuery = mDatabase_events.orderByChild("title").startAt(searchField).endAt(searchField+"\uf8ff");

            //FUNCIO PER OMPLIR EL RECYCLER AMB ELS ESDEVENIMENTS
            readEvents(new MainActivity.GetEventsCallback() {
                @Override
                public void onCallback(List<HashMap<String,EventModel>> events) {
                    EventsAdapter adapter = new EventsAdapter(events);
                    searchEvent_eventList_recycler.setAdapter(adapter);
                }
            });
        }

    }

    //ADAPTER NECESSARI PER EL VIEWHOLDER
    class EventsAdapter extends RecyclerView.Adapter {

        private List<HashMap<String,EventModel>> models = new ArrayList<>();

        public EventsAdapter(final List<HashMap<String,EventModel>> viewModels) {
            if (viewModels != null) {
                this.models.addAll(viewModels);
            }
        }
        @Override
        public RecyclerView.ViewHolder onCreateViewHolder(final ViewGroup parent, final int viewType) {
            final View view = LayoutInflater.from(parent.getContext()).inflate(viewType, parent, false);
            return new SearchEventFragment.EventItemViewHolder(view);
        }
        @Override
        public void onBindViewHolder(final RecyclerView.ViewHolder holder, final int position) {
            ((SearchEventFragment.EventItemViewHolder) holder).bindData(models.get(position));
        }

        @Override
        public int getItemCount() {
            return models.size();
        }


        @Override
        public int getItemViewType(final int position) {
            return R.layout.item_event;
        }
    }

    //CLASSE VIEWHOLDER PER EMPLENAR ELS LAYOUTS ITEM_EVENT
    public class EventItemViewHolder extends RecyclerView.ViewHolder{

        TextView event_title = itemView.findViewById(R.id.itemEvent_title);
        TextView event_locationName = itemView.findViewById(R.id.itemEvent_locationName);
        TextView event_date = itemView.findViewById(R.id.itemEvent_date);
        ImageView event_image = itemView.findViewById(R.id.itemEvent_image);

        public EventItemViewHolder(View itemView){
            super(itemView);
            View mView = itemView;
        }

        public void bindData(final HashMap<String,EventModel> event) {
            event_title.setText(event.values().iterator().next().getTitle());
            event_locationName.setText(event.values().iterator().next().getLocationName());

            SimpleDateFormat sfd = new SimpleDateFormat("dd/MM/yyyy", Locale.FRANCE);
            String dateTimeString = sfd.format(new Date(event.values().iterator().next().getEventTimestamp()));
            event_date.setText(dateTimeString);

            Picasso.with(getContext()).load(event.values().iterator().next().getImage()).into(event_image);

            itemView.setOnClickListener(new View.OnClickListener() {
                @Override
                public void onClick(View v) {
                    Intent searchEvent_viewEvent_Intent = new Intent(getActivity(), ViewEventActivity.class);
                    searchEvent_viewEvent_Intent.putExtra("event_fireBaseKey", event.keySet().iterator().next());
                    startActivity(searchEvent_viewEvent_Intent);
                }
            });
        }
    }

    //CALLBACK PER PODER EMPLENAR ELS ESDEVENIMENTS UN COP CARREGATS
    public interface GetEventsCallback {
        void onCallback(List<HashMap<String,EventModel>> events);
    }

    //FUNCIO QUE RECORRE ELS ESDEVENIMENTS I LES RETORNA AL CALLBACK
    public void readEvents(final MainActivity.GetEventsCallback getEventsCallback) {
        mQuery.addListenerForSingleValueEvent(new ValueEventListener() {

            @Override
            public void onDataChange(DataSnapshot dataSnapshot) {
                List<HashMap<String,EventModel>> events = new ArrayList<>();
                for (DataSnapshot snapshot : dataSnapshot.getChildren()) {
                    String eventKey = snapshot.getKey();
                    EventModel event = snapshot.getValue(EventModel.class);
                    HashMap<String, EventModel> hashMap = new HashMap<>();
                    hashMap.put(eventKey, event);
                    events.add(hashMap);
                    if (snapshot.hasChild("usersInEvent")) {
                        if (!snapshot.child("usersInEvent").hasChild(mAuth.getCurrentUser().getUid()) && (!snapshot.child("userId").getValue().toString().equals(mAuth.getCurrentUser().getUid()))) {
                            events.remove(hashMap);
                        }
                    }
                }
                getEventsCallback.onCallback(events);

            }

            @Override
            public void onCancelled(DatabaseError databaseError) {

            }
        });
    }

}
