package com.dartmaker.uponevent;

import android.content.Context;
import android.content.Intent;
import android.support.v7.app.AppCompatActivity;
import android.os.Bundle;
import android.support.v7.widget.LinearLayoutManager;
import android.support.v7.widget.RecyclerView;
import android.support.v7.widget.Toolbar;
import android.text.TextUtils;
import android.view.View;
import android.widget.ImageView;
import android.widget.TextView;

import com.firebase.ui.database.FirebaseRecyclerAdapter;
import com.google.firebase.auth.FirebaseAuth;
import com.google.firebase.database.DataSnapshot;
import com.google.firebase.database.DatabaseError;
import com.google.firebase.database.DatabaseReference;
import com.google.firebase.database.FirebaseDatabase;
import com.google.firebase.database.ValueEventListener;
import com.squareup.picasso.Picasso;

import java.text.SimpleDateFormat;
import java.util.Date;

public class UserListActivity extends AppCompatActivity {

    private String post_fireBaseKey;
    private String event_fireBaseKey;
    private RecyclerView mUserList;
    private DatabaseReference mUserListDatabase;
    private DatabaseReference mUsersDatabase;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_user_list);

        //INICIALITZEM LES REFERENCIES PRINCIPALS
        mUsersDatabase = FirebaseDatabase.getInstance().getReference().child("Users");
        Intent intent = getIntent();
        Toolbar toolbar = findViewById(R.id.toolbar);
        setSupportActionBar(toolbar);

        //RECUPEREM ELS ID DE PUBLICACIO O D'ESDEVENIMENT
        event_fireBaseKey = intent.getStringExtra("event_fireBaseKey");
        post_fireBaseKey = intent.getStringExtra("post_fireBaseKey");

        //SELECCIONEM EL NODE CORRESPONENT EN EN FUNCIO DE SI ES UN ESDEVENIMENT O UNA PUBLICACIO
        if(!TextUtils.isEmpty(post_fireBaseKey)){
            mUserListDatabase = FirebaseDatabase.getInstance().getReference().child("Posts").child(post_fireBaseKey).child("userLikes");
        } else if(!TextUtils.isEmpty(event_fireBaseKey)){
            mUserListDatabase = FirebaseDatabase.getInstance().getReference().child("Events").child(event_fireBaseKey).child("userLikes");
        }

        mUserList = findViewById(R.id.contentUserList_userList_recycler);
        mUserList.setHasFixedSize(true);
        mUserList.setLayoutManager(new LinearLayoutManager(this));
    }

    @Override
    protected void onStart(){
        super.onStart();

        //CREAM UN RECYCLERADAPTER AMB UN MODEL DE TIPUS BOOLEAN PER TAL DE RECORRER SOLAMENT AQUELLES PUBLICACIONS PRESENTS A L'ESDEVENIMENT I OMPLIR POSTERIORMENT ELS LAYOUTS ITEM_POST AMB LES CLAUS LLISTADES
        final FirebaseRecyclerAdapter<Boolean, UserListActivity.UserListItemViewHolder> FBRA = new FirebaseRecyclerAdapter<Boolean, UserListItemViewHolder>(Boolean.class, R.layout.item_user_list, UserListActivity.UserListItemViewHolder.class, mUserListDatabase) {
            @Override
            protected void populateViewHolder(final UserListActivity.UserListItemViewHolder viewHolder, Boolean model, int position) {

                //RECUPEREM LA CLAU DE LA PUBLICACIO
                final String userList_fireBaseKey = getRef(position).getKey();

                //LISTENER DE LA PUBLICACIO
                mUsersDatabase.child(userList_fireBaseKey).addValueEventListener(new ValueEventListener() {
                    @Override
                    public void onDataChange(DataSnapshot dataSnapshot) {
                        UserModel user = dataSnapshot.getValue(UserModel.class);

                        //COMPROVEM QUE NO HAGI SIGUT ELIMINAT
                        if(user != null){

                            //EMPLENEM ELS LAYOUTS A TRAVES DE LA CLASSE VIEWHOLDER EMPRANT EL RECYCLERADAPTER CREAT AMB UN MODEL DE TIPUS BOLEA
                            final String userId = userList_fireBaseKey;

                            viewHolder.setUserName(user.getName());
                            viewHolder.setUserImage(getApplicationContext(), user.getImage());

                            viewHolder.itemView.setOnClickListener(new View.OnClickListener() {
                                @Override
                                public void onClick(View v) {
                                    Intent userList_viewUser_intent = new Intent(UserListActivity.this, ViewUserActivity.class);
                                    userList_viewUser_intent.putExtra("user_fireBaseKey", userList_fireBaseKey);
                                    startActivity(userList_viewUser_intent);
                                }
                            });
                        }
                    }

                    @Override
                    public void onCancelled(DatabaseError databaseError) {

                    }
                });
            }
        };

        mUserList.setAdapter(FBRA);
    }

    //FUNCIO VIEWHOLDER AUXILIAR PER EMPLENAR ELS LAYOUTS ITEM_POST
    public static class UserListItemViewHolder extends RecyclerView.ViewHolder{
        public UserListItemViewHolder(View itemView){
            super(itemView);
            View mView = itemView;
        }

        public void setUserName(String userName){
            TextView post_userName = itemView.findViewById(R.id.itemUserList_userName_textView);
            post_userName.setText(userName);
        }

        public void setUserImage(Context ctx, String userImage){
            ImageView post_userImage = itemView.findViewById(R.id.itemUserList_userImage_imageView);
            Picasso.with(ctx).load(userImage).into(post_userImage);
        }
    }
}
