package com.dartmaker.uponevent;

import android.content.Context;
import android.content.DialogInterface;
import android.content.Intent;
import android.support.v7.app.AlertDialog;
import android.support.v7.app.AppCompatActivity;
import android.os.Bundle;
import android.support.v7.widget.LinearLayoutManager;
import android.support.v7.widget.RecyclerView;
import android.support.v7.widget.Toolbar;
import android.text.TextUtils;
import android.view.View;
import android.widget.ImageView;
import android.widget.TextView;

import com.firebase.ui.database.FirebaseRecyclerAdapter;
import com.google.firebase.auth.FirebaseAuth;
import com.google.firebase.database.DataSnapshot;
import com.google.firebase.database.DatabaseError;
import com.google.firebase.database.DatabaseReference;
import com.google.firebase.database.FirebaseDatabase;
import com.google.firebase.database.ValueEventListener;
import com.squareup.picasso.Picasso;

import java.util.ArrayList;
import java.util.List;

public class UserListSelectableActivity extends AppCompatActivity {

    private String post_fireBaseKey;
    private String event_fireBaseKey;
    private RecyclerView mUserList;
    private DatabaseReference mUsersEventDatabase;
    private DatabaseReference mUserListDatabase;
    private DatabaseReference mUsersDatabase;
    private ArrayList<String> userIds;
    private FirebaseAuth mAuth;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_user_list_selectable);

        //INICIALITZEM LES REFERENCIES PRINCIPALS
        mUsersDatabase = FirebaseDatabase.getInstance().getReference().child("Users");
        Intent intent = getIntent();
        Toolbar toolbar = findViewById(R.id.toolbar);
        setSupportActionBar(toolbar);
        mAuth = FirebaseAuth.getInstance();

        //RECUPEREM ELS ID DE PUBLICACIO O D'ESDEVENIMENT
        event_fireBaseKey = intent.getStringExtra("event_fireBaseKey");
        post_fireBaseKey = intent.getStringExtra("post_fireBaseKey");
        userIds = intent.getStringArrayListExtra("userIds");

        mUserListDatabase = FirebaseDatabase.getInstance().getReference().child("Users").child(mAuth.getCurrentUser().getUid()).child("usersFollowed");

        //SELECCIONEM EL NODE CORRESPONENT
        if (!TextUtils.isEmpty(event_fireBaseKey)) {
            mUsersEventDatabase = FirebaseDatabase.getInstance().getReference().child("Events").child(event_fireBaseKey).child("usersInEvent");
        }

        mUserList = findViewById(R.id.contentUserListSelectable_userList_recycler);
        mUserList.setHasFixedSize(true);
        mUserList.setLayoutManager(new LinearLayoutManager(this));
    }

    @Override
    protected void onStart() {
        super.onStart();

        //CREAM UN RECYCLERADAPTER AMB UN MODEL DE TIPUS BOOLEAN PER TAL DE RECORRER SOLAMENT AQUELLS USUARIS A QUI HEM FET FOLLOW I OMPLIR POSTERIORMENT ELS LAYOUTS ITEM_USER_LIST AMB LES CLAUS LLISTADES
        final FirebaseRecyclerAdapter<Boolean, UserListSelectableActivity.UserListItemViewHolder> FBRA = new FirebaseRecyclerAdapter<Boolean, UserListItemViewHolder>(Boolean.class, R.layout.item_user_list, UserListSelectableActivity.UserListItemViewHolder.class, mUserListDatabase) {
            @Override
            protected void populateViewHolder(final UserListSelectableActivity.UserListItemViewHolder viewHolder, Boolean model, int position) {

                //RECUPEREM LA CLAU DE LA PUBLICACIO
                final String userList_fireBaseKey = getRef(position).getKey();

                //LISTENER DE L'USUARI
                mUsersDatabase.child(userList_fireBaseKey).addValueEventListener(new ValueEventListener() {
                    @Override
                    public void onDataChange(DataSnapshot dataSnapshot) {
                        UserModel user = dataSnapshot.getValue(UserModel.class);

                        //COMPROVEM QUE NO HAGI SIGUT ELIMINAT
                        if (user != null) {

                            //EMPLENEM ELS LAYOUTS A TRAVES DE LA CLASSE VIEWHOLDER EMPRANT EL RECYCLERADAPTER CREAT AMB UN MODEL DE TIPUS BOLEA
                            final String userId = userList_fireBaseKey;

                            viewHolder.setUserName(user.getName());
                            viewHolder.setUserImage(getApplicationContext(), user.getImage());

                            if (!TextUtils.isEmpty(event_fireBaseKey)) {
                                mUsersEventDatabase.addValueEventListener(new ValueEventListener() {
                                    @Override
                                    public void onDataChange(DataSnapshot dataSnapshot) {
                                        if (dataSnapshot.hasChild(userId)) {
                                            viewHolder.itemView.setBackgroundColor(getResources().getColor(R.color.green_sea));
                                        } else {
                                            viewHolder.itemView.setBackgroundColor(getResources().getColor(android.R.color.white));
                                        }
                                    }

                                    @Override
                                    public void onCancelled(DatabaseError databaseError) {

                                    }
                                });
                            } else if (!userIds.isEmpty()) {
                                if (userIds.contains(userId)) {
                                    viewHolder.itemView.setBackgroundColor(getResources().getColor(R.color.green_sea));
                                } else {
                                    viewHolder.itemView.setBackgroundColor(getResources().getColor(android.R.color.white));
                                }
                            }

                            viewHolder.itemView.setOnClickListener(new View.OnClickListener() {
                                @Override
                                public void onClick(View v) {
                                    if (!TextUtils.isEmpty(event_fireBaseKey)) {
                                        mUsersEventDatabase.addListenerForSingleValueEvent(new ValueEventListener() {
                                            @Override
                                            public void onDataChange(DataSnapshot dataSnapshot) {
                                                if (dataSnapshot.hasChild(userId)) {
                                                    mUsersEventDatabase.child(userId).removeValue();
                                                    viewHolder.itemView.setBackgroundColor(getResources().getColor(android.R.color.white));
                                                } else {
                                                    mUsersEventDatabase.child(userId).setValue(true);
                                                    viewHolder.itemView.setBackgroundColor(getResources().getColor(R.color.green_sea));
                                                }
                                            }

                                            @Override
                                            public void onCancelled(DatabaseError databaseError) {

                                            }
                                        });
                                    } else {
                                        if (userIds.contains(userId)) {
                                            userIds.remove(userId);
                                            viewHolder.itemView.setBackgroundColor(getResources().getColor(android.R.color.white));
                                        } else {
                                            userIds.add(userId);
                                            viewHolder.itemView.setBackgroundColor(getResources().getColor(R.color.green_sea));
                                        }
                                    }
                                }
                            });
                        }
                    }

                    @Override
                    public void onCancelled(DatabaseError databaseError) {

                    }
                });
            }
        };

        mUserList.setAdapter(FBRA);
    }

    //FUNCIO VIEWHOLDER AUXILIAR PER EMPLENAR ELS LAYOUTS ITEM_POST
    public static class UserListItemViewHolder extends RecyclerView.ViewHolder {
        public UserListItemViewHolder(View itemView) {
            super(itemView);
            View mView = itemView;
        }

        public void setUserName(String userName) {
            TextView post_userName = itemView.findViewById(R.id.itemUserList_userName_textView);
            post_userName.setText(userName);
        }

        public void setUserImage(Context ctx, String userImage) {
            ImageView post_userImage = itemView.findViewById(R.id.itemUserList_userImage_imageView);
            Picasso.with(ctx).load(userImage).into(post_userImage);
        }
    }

    public void successButtonClicked(View view) {
        Intent intent = getIntent();
        setResult(RESULT_OK, intent);
        if (TextUtils.isEmpty(event_fireBaseKey)) {
            intent.putStringArrayListExtra("userIds", userIds);
        }
        finish();
    }

    public void publicButtonClicked(View view) {

        new AlertDialog.Builder(this)
                .setIcon(android.R.drawable.ic_dialog_alert)
                .setTitle(getResources().getString(R.string.Make_public))
                .setMessage(getResources().getString(R.string.Confirm_make_public))
                .setPositiveButton(getResources().getString(R.string.Yes), new DialogInterface.OnClickListener() {
                    @Override
                    public void onClick(DialogInterface dialog, int which) {
                        //L'USUARI HA CONFIRMAT LA PUBLICACIO
                        Intent intent = getIntent();
                        setResult(RESULT_OK, intent);
                        if (!TextUtils.isEmpty(event_fireBaseKey)) {
                            mUsersEventDatabase.removeValue();
                        } else {
                            userIds.clear();
                            intent.putStringArrayListExtra("userIds", userIds);
                        }
                        finish();
                    }
                })
                .setNegativeButton(getResources().getString(R.string.No), null)
                .show();
    }
}