package com.dartmaker.uponevent;

import android.content.Context;
import android.content.Intent;
import android.support.design.widget.CollapsingToolbarLayout;
import android.support.v7.app.AppCompatActivity;
import android.os.Bundle;
import android.support.v7.widget.LinearLayoutManager;
import android.support.v7.widget.RecyclerView;
import android.support.v7.widget.Toolbar;
import android.view.LayoutInflater;
import android.view.Menu;
import android.view.MenuItem;
import android.view.View;
import android.view.ViewGroup;
import android.widget.ImageView;
import android.widget.TextView;

import com.firebase.ui.database.FirebaseRecyclerAdapter;
import com.google.android.gms.location.places.ui.PlacePicker;
import com.google.firebase.auth.FirebaseAuth;
import com.google.firebase.database.DataSnapshot;
import com.google.firebase.database.DatabaseError;
import com.google.firebase.database.DatabaseReference;
import com.google.firebase.database.FirebaseDatabase;
import com.google.firebase.database.ValueEventListener;
import com.squareup.picasso.Picasso;

import java.security.Timestamp;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Locale;

public class ViewEventActivity extends AppCompatActivity {
    private String event_fireBaseKey;
    private DatabaseReference mDatabase;
    private DatabaseReference mDatabase_eventPosts;
    private DatabaseReference mUsersDatabase;
    private DatabaseReference mDatabasePosts;
    private FirebaseAuth mAuth;

    private ImageView viewEvent_eventImage;
    private RecyclerView mUponEventList;
    private LinearLayoutManager layoutManager;
    private CollapsingToolbarLayout collapsingToolbar;
    private Menu menu;
    private int position;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_view_event);

        //INICIALITZEM LES REFERENCIES PRINCIPALS
        Intent intent = getIntent();
        event_fireBaseKey = intent.getStringExtra("event_fireBaseKey");
        mDatabase = FirebaseDatabase.getInstance().getReference().child("Events").child(event_fireBaseKey);
        mDatabasePosts = FirebaseDatabase.getInstance().getReference().child("Posts");
        mUsersDatabase = FirebaseDatabase.getInstance().getReference().child("Users");
        mAuth = FirebaseAuth.getInstance();

        //INICIALITZEM LES VARIABLES
        final Toolbar toolbar = findViewById(R.id.viewEvent_toolbar_toolbar);
        setSupportActionBar(toolbar);
        getSupportActionBar().setDisplayHomeAsUpEnabled(false);
        collapsingToolbar = findViewById(R.id.viewEvent_collapsingToolbar_collapsingToolbarLayout);
        collapsingToolbar.setContentScrimColor(getResources().getColor(R.color.colorPrimary));
        collapsingToolbar.setStatusBarScrimColor(getResources().getColor(R.color.cardview_light_background));
        viewEvent_eventImage = findViewById(R.id.viewEvent_eventImage);
        position = 0;

        //Recuperem la recyclerview on anira el llistat de publicacions
        mUponEventList = findViewById(R.id.viewEvent_postList_recyclerView);
        layoutManager = new LinearLayoutManager(this);
        mUponEventList.setLayoutManager(layoutManager);
        mUponEventList.setHasFixedSize(true);
        mUponEventList.setNestedScrollingEnabled(true);
    }

    @Override
    protected void onStart() {
        super.onStart();

        //LISTENER DE L'ESDEVENIMENT
        ValueEventListener eventListener = new ValueEventListener() {
            @Override
            public void onDataChange(DataSnapshot dataSnapshot) {
                //EN PRIMER LLOC RECUPEREM LES DADES DE L'ESDEVENIMENT I OMPLIM LA INFORMACIO DE LA CAPÇALERA
                EventModel event = dataSnapshot.getValue(EventModel.class);
                collapsingToolbar.setTitle(event.getTitle());
                collapsingToolbar.setOnClickListener(new View.OnClickListener() {
                    @Override
                    public void onClick(View v) {
                        Intent viewEvent_imageFullScreen = new Intent(ViewEventActivity.this, ImageFullScreenActivity.class);
                        viewEvent_imageFullScreen.putExtra("event_fireBaseKey", event_fireBaseKey);
                        startActivity(viewEvent_imageFullScreen);
                    }
                });
                Picasso.with(getApplicationContext()).load(event.getImage()).into(viewEvent_eventImage);

                //COMPROVEM SI L'ESDEVENIMENT TE PUBLICACIONS A TRAVES DEL NODE POSTS DE DINTRE DE L'ESDEVENIMENT (LLISTAT DE CLAUS AMB BOLEANS)
                if (dataSnapshot.hasChild("posts")) {
                    mDatabase_eventPosts = mDatabase.child("posts");

                    //FUNCIO PER OMPLIR EL RECYCLER AMB ELS ESDEVENIMENTS
                    readPosts(new ViewEventActivity.GetPostsCallback() {
                        @Override
                        public void onCallback(List<HashMap<String,Boolean>> posts) {
                            ViewEventActivity.PostsAdapter adapter = new ViewEventActivity.PostsAdapter(posts);
                            mUponEventList.setAdapter(adapter);
                            mUponEventList.scrollToPosition(position);
                        }
                    });

                }
            }

            @Override
            public void onCancelled(DatabaseError databaseError) {

            }
        };

        mDatabase.addValueEventListener(eventListener);

    }

    //SALTEM A LA ACTIVITY DE EVENTINFO PASSANT LA CLAU DE L'ESDEVENIMENT
    public void viewEvent_eventInfoButtonClicked() {
        Intent viewEvent_viewEventInfo_intent = new Intent(ViewEventActivity.this, ViewEventInfoActivity.class);
        viewEvent_viewEventInfo_intent.putExtra("event_fireBaseKey", event_fireBaseKey);
        startActivity(viewEvent_viewEventInfo_intent);
    }

    //SALTEM A LA ACTIVITY NEW POST PASSANT LA CLAU DE L'ESDEVENIMENT
    public void viewEvent_newPostButtonClicked() {
        Intent viewEvent_post_intent = new Intent(ViewEventActivity.this, NewPostActivity.class);
        viewEvent_post_intent.putExtra("event_fireBaseKey", event_fireBaseKey);
        startActivity(viewEvent_post_intent);
    }

    //FUNCIO PER GENERAR EL MENU PROPI DE LA ACTIVITY VIEW EVENT
    @Override
    public boolean onCreateOptionsMenu(final Menu menu) {
        this.menu = menu;
        //CREAM EL MENU
        getMenuInflater().inflate(R.menu.menu_view_event, menu);
        return true;
    }

    //CREAM EL LISTENER DEL MENU
    @Override
    public boolean onOptionsItemSelected(MenuItem item) {
        switch (item.getItemId()) {
            case R.id.menuViewEvent_newPost_item:
                viewEvent_newPostButtonClicked();
                return true;
            case R.id.menuViewEvent_eventInfo_item:
                viewEvent_eventInfoButtonClicked();
                return true;
        }
        return super.onOptionsItemSelected(item);
    }

    //ADAPTER NECESSARI PER EL VIEWHOLDER
    class PostsAdapter extends RecyclerView.Adapter {

        private List<HashMap<String, Boolean>> models = new ArrayList<>();

        public PostsAdapter(final List<HashMap<String, Boolean>> viewModels) {
            if (viewModels != null) {
                this.models.addAll(viewModels);
            }
        }

        @Override
        public RecyclerView.ViewHolder onCreateViewHolder(final ViewGroup parent, final int viewType) {
            final View view = LayoutInflater.from(parent.getContext()).inflate(viewType, parent, false);
            return new ViewEventActivity.PostItemViewHolder(view);
        }

        @Override
        public void onBindViewHolder(final RecyclerView.ViewHolder holder, final int position) {
            ((ViewEventActivity.PostItemViewHolder) holder).bindData(models.get(position));
        }

        @Override
        public int getItemCount() {
            return models.size();
        }


        @Override
        public int getItemViewType(final int position) {
            return R.layout.item_post;
        }
    }

    //CLASSE VIEWHOLDER PER EMPLENAR ELS LAYOUTS ITEM_EVENT
    public class PostItemViewHolder extends RecyclerView.ViewHolder {

        TextView post_text = itemView.findViewById(R.id.itemPost_text_textView);
        ImageView post_image = itemView.findViewById(R.id.itemPost_image_imageView);
        TextView post_userName = itemView.findViewById(R.id.itemPost_userName_textView);
        ImageView post_userImage = itemView.findViewById(R.id.itemPost_userImage_imageView);
        TextView itemPost_datetime_textView = itemView.findViewById(R.id.itemPost_timestamp_textView);
        TextView post_numLikes = itemView.findViewById(R.id.itemPost_numLikes_textView);
        TextView post_userNameBold = itemView.findViewById(R.id.itemPost_userNameBold_textView);
        TextView post_dateDiff = itemView.findViewById(R.id.itemPost_dateDiff_textView);
        TextView post_numComments = itemView.findViewById(R.id.itemPost_numComments_textView);

        public PostItemViewHolder(View itemView) {
            super(itemView);
            View mView = itemView;
        }

        public void bindData(final HashMap<String, Boolean> post) {

            final String post_fireBaseKey = post.keySet().iterator().next();

            mDatabasePosts.child(post_fireBaseKey).addValueEventListener(new ValueEventListener() {
                @Override
                public void onDataChange(DataSnapshot dataSnapshot) {
                    PostModel post = dataSnapshot.getValue(PostModel.class);

                    //COMPROVEM QUE NO HAGI SIGUT ELIMINAT

                    if (post != null) {

                        //EMPLENEM ELS LAYOUTS A TRAVES DE LA CLASSE VIEWHOLDER EMPRANT EL RECYCLERADAPTER CREAT AMB UN MODEL DE TIPUS BOLEA
                        final String userId = post.getUserId();
                        post_text.setText(post.getText());
                        Picasso.with(getApplicationContext()).load(post.getImage()).into(post_image);

                        SimpleDateFormat sfd = new SimpleDateFormat("dd/MM/yyyy HH:mm");
                        String dateTimeString = sfd.format(new Date(post.getTimestamp())).toString();
                        itemPost_datetime_textView.setText(dateTimeString);

                        long numComments = dataSnapshot.child("comments").getChildrenCount();
                        if (numComments == 0) {
                            post_numComments.setVisibility(View.INVISIBLE);
                            post_numComments.setHeight(0);
                        } else {
                            post_numComments.setText(getResources().getString(R.string.Read_the) + " " + String.valueOf(numComments) + " " + getResources().getString(R.string.Comments));
                        }

                        long numLikes = dataSnapshot.child("userLikes").getChildrenCount();
                        if (numLikes == 0) {
                            post_numLikes.setVisibility(View.INVISIBLE);
                            post_numLikes.setHeight(0);
                        } else {

                            post_numLikes.setText(String.valueOf(numLikes) + " " + getResources().getString(R.string.Like));
                        }

                        String difference = ApplicationUtils.getDateDiff(post.getTimestamp(), System.currentTimeMillis());
                        post_dateDiff.setText(difference);

                        itemView.findViewById(R.id.itemPost_image_imageView).setOnClickListener(new View.OnClickListener() {
                            @Override
                            public void onClick(View v) {
                                position = getAdapterPosition();
                                Intent viewEvent_viewPost_Intent = new Intent(ViewEventActivity.this, ViewPostActivity.class);
                                viewEvent_viewPost_Intent.putExtra("post_fireBaseKey", post_fireBaseKey);
                                startActivity(viewEvent_viewPost_Intent);
                            }
                        });

                        itemView.findViewById(R.id.itemPost_text_linearLayout).setOnClickListener(new View.OnClickListener() {
                            @Override
                            public void onClick(View v) {
                                position = getAdapterPosition();
                                Intent viewEvent_viewPost_Intent = new Intent(ViewEventActivity.this, ViewPostActivity.class);
                                viewEvent_viewPost_Intent.putExtra("post_fireBaseKey", post_fireBaseKey);
                                startActivity(viewEvent_viewPost_Intent);
                            }
                        });

                        itemView.findViewById(R.id.itemPost_numComments_textView).setOnClickListener(new View.OnClickListener() {
                            @Override
                            public void onClick(View v) {
                                position = getAdapterPosition();
                                Intent itemPost_comment_intent = new Intent(ViewEventActivity.this, CommentActivity.class);
                                itemPost_comment_intent.putExtra("post_fireBaseKey", post_fireBaseKey);
                                startActivity(itemPost_comment_intent);
                            }
                        });

                        itemView.findViewById(R.id.itemPost_numLikes_textView).setOnClickListener(new View.OnClickListener() {
                            @Override
                            public void onClick(View v) {
                                position = getAdapterPosition();
                                Intent itemPost_userList_intent = new Intent(ViewEventActivity.this, UserListActivity.class);
                                itemPost_userList_intent.putExtra("post_fireBaseKey", post_fireBaseKey);
                                startActivity(itemPost_userList_intent);
                            }
                        });

                        itemView.findViewById(R.id.viewPost_userInfo_linearLayout).setOnClickListener(new View.OnClickListener() {
                            @Override
                            public void onClick(View v) {
                                position = getAdapterPosition();
                                Intent main_viewUser_Intent = new Intent(ViewEventActivity.this, ViewUserActivity.class);
                                main_viewUser_Intent.putExtra("user_fireBaseKey", userId);
                                startActivity(main_viewUser_Intent);
                            }
                        });

                        mUsersDatabase.child(post.getUserId()).addValueEventListener(new ValueEventListener() {
                            @Override
                            public void onDataChange(DataSnapshot dataSnapshot) {
                                //EMPLENEM ELS CAMPS D'INFORMACIO D'USUARI DEL LAYOUT ITEM_POST
                                UserModel user = dataSnapshot.getValue(UserModel.class);

                                post_userName.setText(user.getName());
                                post_userNameBold.setText(user.getUsername() + ": ");

                                Picasso.with(getApplicationContext()).load(user.getImage()).into(post_userImage);

                            }

                            @Override
                            public void onCancelled(DatabaseError databaseError) {

                            }
                        });
                    }
                }

                @Override
                public void onCancelled(DatabaseError databaseError) {

                }
            });

        }
    }

    //CALLBACK PER PODER EMPLENAR ELS ESDEVENIMENTS UN COP CARREGATS
    public interface GetPostsCallback {
        void onCallback(List<HashMap<String,Boolean>> posts);
    }

    //FUNCIO QUE RECORRE ELS ESDEVENIMENTS I LES RETORNA AL CALLBACK
    public void readPosts(final ViewEventActivity.GetPostsCallback getPostsCallback) {
        mDatabase_eventPosts.addListenerForSingleValueEvent(new ValueEventListener() {

            @Override
            public void onDataChange(DataSnapshot dataSnapshot) {
                List<HashMap<String,Boolean>> posts = new ArrayList<>();
                for (DataSnapshot snapshot : dataSnapshot.getChildren()) {
                    String postKey = snapshot.getKey();

                    Boolean post = snapshot.getValue(Boolean.class);
                    HashMap<String, Boolean> hashMap = new HashMap<>();
                    hashMap.put(postKey, post);
                    posts.add(hashMap);
                }
                getPostsCallback.onCallback(posts);

            }

            @Override
            public void onCancelled(DatabaseError databaseError) {

            }
        });
    }
}