package com.dartmaker.uponevent;

import android.content.Context;
import android.content.Intent;
import android.graphics.Color;
import android.graphics.drawable.Drawable;
import android.net.Uri;
import android.support.annotation.ColorRes;
import android.support.design.widget.CollapsingToolbarLayout;
import android.support.v4.graphics.drawable.DrawableCompat;
import android.support.v7.app.AppCompatActivity;
import android.os.Bundle;
import android.support.v7.widget.CardView;
import android.support.v7.widget.LinearLayoutManager;
import android.support.v7.widget.RecyclerView;
import android.support.v7.widget.Toolbar;
import android.view.Menu;
import android.view.MenuItem;
import android.view.View;
import android.webkit.WebChromeClient;
import android.webkit.WebView;
import android.widget.ImageView;
import android.widget.LinearLayout;
import android.widget.TextView;
import android.widget.Toast;

import com.firebase.ui.database.FirebaseRecyclerAdapter;
import com.google.android.gms.maps.GoogleMap;
import com.google.android.gms.maps.MapFragment;
import com.google.android.gms.maps.OnMapReadyCallback;
import com.google.firebase.auth.FirebaseAuth;
import com.google.firebase.database.DataSnapshot;
import com.google.firebase.database.DatabaseError;
import com.google.firebase.database.DatabaseReference;
import com.google.firebase.database.FirebaseDatabase;
import com.google.firebase.database.ValueEventListener;
import com.squareup.picasso.Picasso;

import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.Locale;

public class ViewEventInfoActivity extends AppCompatActivity {
    private String event_fireBaseKey;
    private DatabaseReference mDatabase;
    private DatabaseReference mDatabase_eventPosts;
    private DatabaseReference mDatabase_eventGuests;
    private DatabaseReference mUsersDatabase;
    private DatabaseReference mDatabasePosts;
    private FirebaseAuth mAuth;
    private ApplicationUtils applicationUtils;

    private ImageView viewEventInfo_eventImage;
    private TextView viewEventInfo_user_textView;
    private TextView viewEventInfo_location_textView;
    private TextView viewEventInfo_date_textView;
    private TextView viewEventInfo_users_textView;
    private ImageView viewEventInfo_users_icon;
    private TextView viewEventInfo_likes_textView;
    private LinearLayout viewEventInfo_likes_linearLayout;
    private TextView viewEventInfo_comments_textView;
    private LinearLayout viewEventInfo_comments_linearLayout;
    private WebView viewEventInfo_desc_webView;
    private GoogleMap googleMap;
    private CardView postsListCardView;
    private RecyclerView mImagesList;
    private CardView guestsListCardView;
    private RecyclerView mGuestsList;
    private CollapsingToolbarLayout collapsingToolbar;
    private Menu menu;


    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_view_event_info);

        //INICIALITZEM LES REFERENCIES PRINCIPALS
        Intent intent = getIntent();
        event_fireBaseKey = intent.getStringExtra("event_fireBaseKey");
        mDatabase = FirebaseDatabase.getInstance().getReference().child("Events").child(event_fireBaseKey);
        mDatabasePosts = FirebaseDatabase.getInstance().getReference().child("Posts");
        mUsersDatabase = FirebaseDatabase.getInstance().getReference().child("Users");
        mAuth = FirebaseAuth.getInstance();
        applicationUtils = new ApplicationUtils();

        //INICIALITZEM LES VARIABLES
        final Toolbar toolbar = findViewById(R.id.viewEventInfo_toolbar_toolbar);
        setSupportActionBar(toolbar);
        getSupportActionBar().setDisplayHomeAsUpEnabled(false);
        collapsingToolbar = findViewById(R.id.viewEventInfo_collapsingToolbar_collapsingToolbarLayout);
        collapsingToolbar.setContentScrimColor(getResources().getColor(R.color.colorPrimary));
        collapsingToolbar.setStatusBarScrimColor(getResources().getColor(R.color.cardview_light_background));
        viewEventInfo_eventImage = findViewById(R.id.viewEventInfo_eventImage);
        viewEventInfo_user_textView = findViewById(R.id.viewEventInfo_user_textView);
        viewEventInfo_location_textView = findViewById(R.id.viewEventInfo_location_textView);
        viewEventInfo_date_textView = findViewById(R.id.viewEventInfo_date_textView);
        viewEventInfo_users_textView = findViewById(R.id.viewEventInfo_users_textView);
        viewEventInfo_users_icon = findViewById(R.id.viewEventInfo_users_icon);
        viewEventInfo_likes_textView = findViewById(R.id.viewEventInfo_likes_textView);
        viewEventInfo_likes_linearLayout = findViewById(R.id.viewEventInfo_likes_linearLayout);
        viewEventInfo_comments_textView = findViewById(R.id.viewEventInfo_comments_textView);
        viewEventInfo_comments_linearLayout = findViewById(R.id.viewEventInfo_comments_linearLayout);
        viewEventInfo_desc_webView = findViewById(R.id.viewEventInfo_desc_webView);


        //Recuperem la recyclerview on anira el llistat de publicacions
        postsListCardView = findViewById(R.id.viewEventInfo_galley_cardView);
        mImagesList = findViewById(R.id.viewEventInfo_galley_recyclerView);
        mImagesList.setLayoutManager(new LinearLayoutManager(this, LinearLayoutManager.HORIZONTAL, false));
        mImagesList.setHasFixedSize(true);
        mImagesList.setNestedScrollingEnabled(true);

        //Recuperem la recyclerview on anira el llistat de convidats
        guestsListCardView = findViewById(R.id.viewEventInfo_guestsGalley_cardView);
        mGuestsList = findViewById(R.id.viewEventInfo_guestsGalley_recyclerView);
        mGuestsList.setLayoutManager(new LinearLayoutManager(this, LinearLayoutManager.HORIZONTAL, false));
        mGuestsList.setHasFixedSize(true);
        mGuestsList.setNestedScrollingEnabled(true);
    }

    @Override
    protected void onStart() {
        super.onStart();

        //LISTENER DE L'ESDEVENIMENT
        ValueEventListener eventListener = new ValueEventListener() {
            @Override
            public void onDataChange(DataSnapshot dataSnapshot) {
                //EN PRIMER LLOC RECUPEREM LES DADES DE L'ESDEVENIMENT I OMPLIM LA INFORMACIO DE LA CAPÇALERA
                final EventModel event = dataSnapshot.getValue(EventModel.class);
                collapsingToolbar.setTitle(event.getTitle());
                collapsingToolbar.setOnClickListener(new View.OnClickListener() {
                    @Override
                    public void onClick(View v) {
                        Intent viewEventInfo_imageFullScreen = new Intent(ViewEventInfoActivity.this, ImageFullScreenActivity.class);
                        viewEventInfo_imageFullScreen.putExtra("event_fireBaseKey", event_fireBaseKey);
                        startActivity(viewEventInfo_imageFullScreen);
                    }
                });
                Picasso.with(getApplicationContext()).load(event.getImage()).into(viewEventInfo_eventImage);
                viewEventInfo_location_textView.setText(event.getLocationName());

                SimpleDateFormat sfd = new SimpleDateFormat("dd/MM/yyyy", Locale.FRANCE);
                String dateTimeString = sfd.format(new Date(event.getEventTimestamp()));
                viewEventInfo_date_textView.setText(dateTimeString);

                if(!dataSnapshot.hasChild("usersInEvent")){
                    viewEventInfo_users_icon.setImageDrawable(getResources().getDrawable(R.drawable.ic_lock_open_white_35dp));
                    viewEventInfo_users_textView.setText(getResources().getString(R.string.Public_event));
                }

                long numComments = 0;
                if(dataSnapshot.hasChild("comments")){
                    numComments = dataSnapshot.child("comments").getChildrenCount();
                    viewEventInfo_comments_linearLayout.setOnClickListener(new View.OnClickListener() {
                        @Override
                        public void onClick(View v) {
                            Intent viewEventInfo_comment_intent = new Intent(ViewEventInfoActivity.this, CommentActivity.class);
                            viewEventInfo_comment_intent.putExtra("event_fireBaseKey", event_fireBaseKey);
                            startActivity(viewEventInfo_comment_intent);
                        }
                    });
                }
                viewEventInfo_comments_textView.setText(numComments + " " + getResources().getString(R.string.Comments));

                long numLikes = 0;
                if(dataSnapshot.hasChild("userLikes")){
                    numLikes = dataSnapshot.child("userLikes").getChildrenCount();
                    viewEventInfo_likes_linearLayout.setOnClickListener(new View.OnClickListener() {
                        @Override
                        public void onClick(View v) {
                            Intent viewEventInfo_userList_intent = new Intent(ViewEventInfoActivity.this, UserListActivity.class);
                            viewEventInfo_userList_intent.putExtra("event_fireBaseKey", event_fireBaseKey);
                            startActivity(viewEventInfo_userList_intent);
                        }
                    });
                }
                viewEventInfo_likes_textView.setText(numLikes + " " + getResources().getString(R.string.Like));

                String html_data = "<style>img{max-width:100%;height:auto;} iframe{width:100%;}</style> ";
                html_data += event.getDesc();
                viewEventInfo_desc_webView.getSettings().setBuiltInZoomControls(true);
                viewEventInfo_desc_webView.setBackgroundColor(Color.TRANSPARENT);
                viewEventInfo_desc_webView.setWebChromeClient(new WebChromeClient());
                viewEventInfo_desc_webView.loadData(html_data, "text/html; charset=UTF-8", null);

                mUsersDatabase.child(event.getUserId()).addValueEventListener(new ValueEventListener() {
                    @Override
                    public void onDataChange(DataSnapshot dataSnapshot) {
                        //EMPLENEM ELS CAMPS D'INFORMACIO D'USUARI
                        UserModel user = dataSnapshot.getValue(UserModel.class);
                        viewEventInfo_user_textView.setText(user.getUsername());
                    }

                    @Override
                    public void onCancelled(DatabaseError databaseError) {

                    }
                });

                //COMPROVEM SI L'ESDEVENIMENT TE PUBLICACIONS A TRAVES DEL NODE POSTS DE DINTRE DE L'ESDEVENIMENT (LLISTAT DE CLAUS AMB BOLEANS)
                if (dataSnapshot.hasChild("posts")) {
                    mDatabase_eventPosts = mDatabase.child("posts");

                    //CREAM UN RECYCLERADAPTER AMB UN MODEL DE TIPUS BOOLEAN PER TAL DE RECORRER SOLAMENT AQUELLES PUBLICACIONS PRESENTS A L'ESDEVENIMENT I OMPLIR POSTERIORMENT ELS LAYOUTS ITEM_POST AMB LES CLAUS LLISTADES
                    final FirebaseRecyclerAdapter<Boolean, ViewEventInfoActivity.ImageItemViewHolder> FBRA = new FirebaseRecyclerAdapter<Boolean, ViewEventInfoActivity.ImageItemViewHolder>(Boolean.class, R.layout.item_image, ViewEventInfoActivity.ImageItemViewHolder.class, mDatabase_eventPosts) {
                        @Override
                        protected void populateViewHolder(final ViewEventInfoActivity.ImageItemViewHolder viewHolder, Boolean model, int position) {

                            //RECUPEREM LA CLAU DE LA PUBLICACIO
                            final String post_fireBaseKey = getRef(position).getKey();

                            //LISTENER DE LA PUBLICACIO
                            mDatabasePosts.child(post_fireBaseKey).addValueEventListener(new ValueEventListener() {
                                @Override
                                public void onDataChange(DataSnapshot dataSnapshot) {
                                    PostModel post = dataSnapshot.getValue(PostModel.class);

                                    //COMPROVEM QUE NO HAGI SIGUT ELIMINAT
                                    if (post != null) {
                                        //EMPLENEM ELS LAYOUTS A TRAVES DE LA CLASSE VIEWHOLDER EMPRANT EL RECYCLERADAPTER CREAT AMB UN MODEL DE TIPUS BOLEA
                                        viewHolder.setImage(getApplicationContext(), post.getImage());
                                        viewHolder.itemView.findViewById(R.id.itemImage_image_imageView).setOnClickListener(new View.OnClickListener() {
                                            @Override
                                            public void onClick(View v) {
                                                Intent viewEventInfo_imageFullScreen_Intent = new Intent(ViewEventInfoActivity.this, ImageFullScreenActivity.class);
                                                viewEventInfo_imageFullScreen_Intent.putExtra("post_fireBaseKey", post_fireBaseKey);
                                                startActivity(viewEventInfo_imageFullScreen_Intent);
                                            }
                                        });
                                    }
                                }

                                @Override
                                public void onCancelled(DatabaseError databaseError) {

                                }
                            });
                        }
                    };

                    mImagesList.setAdapter(FBRA);
                }else{
                    postsListCardView.setVisibility(View.GONE);
                }

                //COMPROVEM SI L'ESDEVENIMENT TE CONVIDATS A TRAVES DEL NODE USERSINEVENT DE DINTRE DE L'ESDEVENIMENT (LLISTAT DE CLAUS AMB BOLEANS)
                if (dataSnapshot.hasChild("usersInEvent")) {
                    mDatabase_eventGuests = mDatabase.child("usersInEvent");

                    //CREAM UN RECYCLERADAPTER AMB UN MODEL DE TIPUS BOOLEAN PER TAL DE RECORRER SOLAMENT AQUELLS USUARIS PRESENTS A L'ESDEVENIMENT I OMPLIR POSTERIORMENT ELS LAYOUTS ITEM_IMAGE AMB LES CLAUS LLISTADES
                    final FirebaseRecyclerAdapter<Boolean, ViewEventInfoActivity.ImageItemViewHolder> FBRAU = new FirebaseRecyclerAdapter<Boolean, ViewEventInfoActivity.ImageItemViewHolder>(Boolean.class, R.layout.item_image, ViewEventInfoActivity.ImageItemViewHolder.class, mDatabase_eventGuests) {
                        @Override
                        protected void populateViewHolder(final ViewEventInfoActivity.ImageItemViewHolder viewHolder, Boolean model, int position) {

                            //RECUPEREM LA CLAU DE L'USUARI
                            final String user_fireBaseKey = getRef(position).getKey();

                            //LISTENER DE LA PUBLICACIO
                            mUsersDatabase.child(user_fireBaseKey).addValueEventListener(new ValueEventListener() {
                                @Override
                                public void onDataChange(DataSnapshot dataSnapshot) {
                                    UserModel user = dataSnapshot.getValue(UserModel.class);

                                    //COMPROVEM QUE NO HAGI SIGUT ELIMINAT
                                    if (user != null) {
                                        //EMPLENEM ELS LAYOUTS A TRAVES DE LA CLASSE VIEWHOLDER EMPRANT EL RECYCLERADAPTER CREAT AMB UN MODEL DE TIPUS BOLEA
                                        viewHolder.setImage(getApplicationContext(), user.getImage());
                                        viewHolder.itemView.findViewById(R.id.itemImage_image_imageView).setOnClickListener(new View.OnClickListener() {
                                            @Override
                                            public void onClick(View v) {
                                                Intent viewEventInfo_viewUser_Intent = new Intent(ViewEventInfoActivity.this, ViewUserActivity.class);
                                                viewEventInfo_viewUser_Intent.putExtra("user_fireBaseKey", user_fireBaseKey);
                                                startActivity(viewEventInfo_viewUser_Intent);
                                            }
                                        });
                                    }
                                }

                                @Override
                                public void onCancelled(DatabaseError databaseError) {

                                }
                            });
                        }
                    };

                    mGuestsList.setAdapter(FBRAU);
                }
                else{
                    guestsListCardView.setVisibility(View.GONE);
                }


                //AQUESTA PART INICIALITZA EL MAPA PER MOSTRAR PER PANTALLA LA UBICACIO
                if (googleMap == null) {
                    MapFragment mapFragment1 = (MapFragment) getFragmentManager().findFragmentById(R.id.viewEventInfo_map_fragment);
                    mapFragment1.getMapAsync(new OnMapReadyCallback() {
                        @Override
                        public void onMapReady(GoogleMap gMap) {
                            googleMap = gMap;
                            if (googleMap == null) {
                                Toast.makeText(getApplicationContext(), getResources().getString(R.string.Map_cannot_be_created), Toast.LENGTH_SHORT).show();
                            } else {
                                // CONFIGUREM EL MAPA
                                googleMap = applicationUtils.configStaticMap(ViewEventInfoActivity.this, googleMap, event.getLatitude(), event.getLongitude());
                            }
                        }
                    });
                }

                findViewById(R.id.viewEventInfo_mapNavigate_Button).setOnClickListener(new View.OnClickListener() {
                    @Override
                    public void onClick(View view) {
                        Intent navigation = new Intent(Intent.ACTION_VIEW, Uri.parse("http://maps.google.com/maps?daddr=" + event.getLatitude() + "," + event.getLongitude()));
                        startActivity(navigation);
                    }
                });

                findViewById(R.id.viewEventInfo_mapView_Button).setOnClickListener(new View.OnClickListener() {
                    @Override
                    public void onClick(View view) {
                        Intent viewEventInfo_maps_intent = new Intent(ViewEventInfoActivity.this, MapsActivity.class);
                        viewEventInfo_maps_intent.putExtra("latitude", event.getLatitude());
                        viewEventInfo_maps_intent.putExtra("longitude", event.getLongitude());
                        viewEventInfo_maps_intent.putExtra("locationName", event.getLocationName());
                        startActivity(viewEventInfo_maps_intent);
                    }
                });

                findViewById(R.id.viewEventInfo_map_linearLayout).setOnClickListener(new View.OnClickListener() {
                    @Override
                    public void onClick(View view) {
                        Intent viewEventInfo_maps_intent = new Intent(ViewEventInfoActivity.this, MapsActivity.class);
                        viewEventInfo_maps_intent.putExtra("latitude", event.getLatitude());
                        viewEventInfo_maps_intent.putExtra("longitude", event.getLongitude());
                        viewEventInfo_maps_intent.putExtra("locationName", event.getLocationName());
                        startActivity(viewEventInfo_maps_intent);
                    }
                });
            }

            @Override
            public void onCancelled(DatabaseError databaseError) {

            }
        };

        mDatabase.addValueEventListener(eventListener);

    }

    //FUNCIO VIEWHOLDER AUXILIAR PER EMPLENAR ELS LAYOUTS ITEM_IMAGE
    public static class ImageItemViewHolder extends RecyclerView.ViewHolder {
        public ImageItemViewHolder(View itemView) {
            super(itemView);
            View mView = itemView;
        }

        public void setImage(Context ctx, String image) {
            ImageView event_image = itemView.findViewById(R.id.itemImage_image_imageView);
            Picasso.with(ctx).load(image).into(event_image);

        }
    }

    //SALTEM A LA ACTIVITY DE COMENTARI PASSANT LA CLAU DE L'ESDEVENIMENT
    public void viewEvent_commentButtonClicked() {
        Intent viewEvent_comment_intent = new Intent(ViewEventInfoActivity.this, CommentActivity.class);
        viewEvent_comment_intent.putExtra("event_fireBaseKey", event_fireBaseKey);
        startActivity(viewEvent_comment_intent);
    }

    // INDIQUEM L'ESDEVENIMENT COM A FAV
    public void viewEvent_favButtonClicked() {
        //LISTENER DE USUARIS QUE HAN FET FAV
        mDatabase.child("userLikes").addListenerForSingleValueEvent(new ValueEventListener() {
            @Override
            public void onDataChange(DataSnapshot dataSnapshot) {
                //SI JA ERA FAV, ELIMINEM EL NODE I CANVIEM LA ICONA A BLANC
                if (dataSnapshot.hasChild(mAuth.getCurrentUser().getUid())) {
                    mDatabase.child("userLikes").child(mAuth.getCurrentUser().getUid()).removeValue();
                    tintMenuIcon(ViewEventInfoActivity.this, menu.findItem(R.id.menuViewEventInfo_fav_item), android.R.color.white);
                } else {
                    //NO ERA FAV, AFEGIM EL NODE AMB LA CLAU D'USUARI I CANVIEM LA ICONA A VERMELL
                    mDatabase.child("userLikes").child(mAuth.getCurrentUser().getUid()).setValue(true);
                    tintMenuIcon(ViewEventInfoActivity.this, menu.findItem(R.id.menuViewEventInfo_fav_item), android.R.color.holo_red_dark);
                }
            }

            @Override
            public void onCancelled(DatabaseError databaseError) {

            }
        });
    }

    //FUNCIO PER GENERAR EL MENU PROPI DE LA ACTIVITY VIEW EVENT
    @Override
    public boolean onCreateOptionsMenu(final Menu menu) {
        this.menu = menu;

        //SELECCIONEM L'USUARI DE L'ESDEVENIMENT
        mDatabase.child("userId").addValueEventListener(new ValueEventListener() {
            @Override
            public void onDataChange(DataSnapshot dataSnapshot) {

                //SI L'USUARI DE L'ESDEVENIMENT ES CORRESPON AMB L'USUARI ACTUAL, TINDRA PERMISOS D'EDICIO, CREAM LA ICONA
                if(dataSnapshot.getValue(String.class).equals(mAuth.getCurrentUser().getUid())){

                    Intent viewEventInfo_editEvent_intent = new Intent(ViewEventInfoActivity.this, EditEventActivity.class);
                    viewEventInfo_editEvent_intent.putExtra("event_fireBaseKey", event_fireBaseKey);
                    menu.add(Menu.NONE,Menu.NONE, Menu.NONE, getResources().getString(R.string.Edit)).setIcon(R.drawable.ic_edit_white_35dp).setIntent(viewEventInfo_editEvent_intent).setShowAsAction(MenuItem.SHOW_AS_ACTION_ALWAYS);
                }
            }

            @Override
            public void onCancelled(DatabaseError databaseError) {

            }
        });

        //CREAM EL MENU I RECUPEREM LES ICONES
        getMenuInflater().inflate(R.menu.menu_view_event_info, menu);
        final MenuItem menuViewEventInfo_fav_item = menu.findItem(R.id.menuViewEventInfo_fav_item);
        if (menuViewEventInfo_fav_item != null) {
            //RECUPEREM EL LLISTAT DE FAV'S
            mDatabase.child("userLikes").addListenerForSingleValueEvent(new ValueEventListener() {
                @Override
                public void onDataChange(DataSnapshot dataSnapshot) {
                    if (dataSnapshot.hasChild(mAuth.getCurrentUser().getUid())) {
                        //SI L'USUARI ACTUAL HA FET LIKE, PINTEM LA ICONA DE VERMELL
                        tintMenuIcon(ViewEventInfoActivity.this, menuViewEventInfo_fav_item, android.R.color.holo_red_dark);
                    } else {
                        // SI L'USUARI ACTUAL NO HA FET LIKE, PINTEM LA ICONA DE BLANC
                        tintMenuIcon(ViewEventInfoActivity.this, menuViewEventInfo_fav_item, android.R.color.white);
                    }
                }

                @Override
                public void onCancelled(DatabaseError databaseError) {

                }
            });

        }

        return true;
    }

    //FUNCIO AUXILIAR PER CANVIAR UN ITEM DEL MENU DE COLOR
    public static void tintMenuIcon(Context context, MenuItem item, @ColorRes int color) {
        Drawable normalDrawable = item.getIcon();
        Drawable wrapDrawable = DrawableCompat.wrap(normalDrawable);
        DrawableCompat.setTint(wrapDrawable, context.getResources().getColor(color));

        item.setIcon(wrapDrawable);
    }

    //CREAM EL LISTENER DEL MENU
    @Override
    public boolean onOptionsItemSelected(MenuItem item) {
        switch (item.getItemId()) {

            case R.id.menuViewEventInfo_fav_item:
                viewEvent_favButtonClicked();
                return true;
            case R.id.menuViewEventInfo_comment_item:
                viewEvent_commentButtonClicked();
                return true;
        }
        return super.onOptionsItemSelected(item);
    }
}