package com.dartmaker.uponevent;

import android.content.Context;
import android.content.DialogInterface;
import android.content.Intent;
import android.graphics.Bitmap;
import android.graphics.PorterDuff;
import android.graphics.drawable.BitmapDrawable;
import android.graphics.drawable.Drawable;
import android.os.Handler;
import android.support.v4.content.ContextCompat;
import android.support.v7.app.AlertDialog;
import android.support.v7.app.AppCompatActivity;
import android.os.Bundle;
import android.support.v7.widget.LinearLayoutManager;
import android.support.v7.widget.RecyclerView;
import android.text.TextUtils;
import android.view.View;
import android.widget.ImageButton;
import android.widget.ImageView;
import android.widget.LinearLayout;
import android.widget.TextView;

import com.firebase.ui.database.FirebaseRecyclerAdapter;
import com.google.firebase.auth.FirebaseAuth;
import com.google.firebase.database.DataSnapshot;
import com.google.firebase.database.DatabaseError;
import com.google.firebase.database.DatabaseReference;
import com.google.firebase.database.FirebaseDatabase;
import com.google.firebase.database.ValueEventListener;
import com.squareup.picasso.Callback;
import com.squareup.picasso.Picasso;
import com.squareup.picasso.Target;

import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.Locale;

public class ViewPostActivity extends AppCompatActivity {

    private String post_fireBaseKey;
    private DatabaseReference mDatabase;
    private DatabaseReference mDatabaseComment;
    private DatabaseReference mDatabaseUsers;
    private FirebaseAuth mAuth;

    private TextView viewPost_postText;
    private TextView viewPost_userNameBold_textView;
    private ImageView viewPost_postImage;
    private ImageButton viewPost_favIcon_imageButton;
    private TextView viewPost_numLikes_textView;
    private TextView viewPost_date_textView;
    private RecyclerView mCommentsList;


    private LinearLayout viewPost_icons_linearLayout;
    private String eventId;
    private String userId;
    private boolean deleteButtonAdded = false;
    ValueEventListener postListener;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_view_post);

        //INICIALITZEM LES REFERENCIES PRINCIPALS
        Intent intent = getIntent();
        post_fireBaseKey = intent.getStringExtra("post_fireBaseKey");
        mDatabase = FirebaseDatabase.getInstance().getReference().child("Posts").child(post_fireBaseKey);
        mDatabaseComment = mDatabase.child("comments");
        mDatabaseUsers = FirebaseDatabase.getInstance().getReference().child("Users");
        mAuth = FirebaseAuth.getInstance();

        //INICIALITZEM LES VARIABLES
        viewPost_postText = findViewById(R.id.viewPost_text_textView);
        viewPost_postImage = findViewById(R.id.viewPost_postImage);
        viewPost_favIcon_imageButton = findViewById(R.id.viewPost_favIcon_imageButton);
        viewPost_icons_linearLayout = findViewById(R.id.viewPost_icons_linearLayout);
        viewPost_userNameBold_textView = findViewById(R.id.viewPost_userNameBold_textView);
        viewPost_numLikes_textView = findViewById(R.id.viewPost_numLikes_textView);
        viewPost_date_textView = findViewById(R.id.viewPost_date_textView);

        mCommentsList = findViewById(R.id.viewPost_commentList_recycler);
        mCommentsList.setHasFixedSize(true);
        mCommentsList.setLayoutManager(new LinearLayoutManager(this));
    }

    @Override
    protected void onStart() {
        super.onStart();

        //LISTENER PER EL PROPI POST
        postListener = new ValueEventListener() {
            @Override
            public void onDataChange(DataSnapshot dataSnapshot) {
                PostModel post = dataSnapshot.getValue(PostModel.class);
                //VALIDEM QUE EL POST NO HAGI SIGUT ELIMINAT
                if(post != null) {
                    eventId = post.getEventId();
                    userId = post.getUserId();

                    long numComments = 0;
                    if(dataSnapshot.hasChild("comments")){
                        numComments = dataSnapshot.child("comments").getChildrenCount();
                    }

                    String dateDiff = ApplicationUtils.getDateDiff(post.getTimestamp(), System.currentTimeMillis());
                    viewPost_date_textView.setText(dateDiff);

                    long numLikes = 0;
                    if(dataSnapshot.hasChild("userLikes")){
                        numLikes = dataSnapshot.child("userLikes").getChildrenCount();
                        viewPost_numLikes_textView.setOnClickListener(new View.OnClickListener() {
                            @Override
                            public void onClick(View v) {
                                Intent viewPost_userList_intent = new Intent(ViewPostActivity.this, UserListActivity.class);
                                viewPost_userList_intent.putExtra("post_fireBaseKey", post_fireBaseKey);
                                startActivity(viewPost_userList_intent);
                            }
                        });
                    }
                    viewPost_numLikes_textView.setText(numLikes + " " + getResources().getString(R.string.Like));

                    mDatabaseUsers.child(userId).addListenerForSingleValueEvent(new ValueEventListener() {
                        @Override
                        public void onDataChange(DataSnapshot dataSnapshot) {
                            UserModel user = dataSnapshot.getValue(UserModel.class);
                            viewPost_userNameBold_textView.setText(user.getUsername() + ": ");
                        }

                        @Override
                        public void onCancelled(DatabaseError databaseError) {

                        }
                    });

                    viewPost_postText.setText(post.getText());


                    final ImageView profile = new ImageView(getApplicationContext());
                    Picasso.with(getApplicationContext()).load(post.getImage()).into(profile, new Callback() {
                        @Override
                        public void onSuccess() {
                            new Handler().postDelayed(new Runnable() {
                                @Override
                                public void run() {

                                    Bitmap innerBitmap = ((BitmapDrawable) profile.getDrawable()).getBitmap();
                                    Bitmap b2 = ApplicationUtils.resizeFullWidthMaxHeight(innerBitmap, 700, getApplicationContext());
                                    viewPost_postImage.setImageBitmap(b2);

                                }
                            }, 100);
                        }

                        @Override
                        public void onError() {

                        }
                    });

                    //SI L'USUARI DEL POST ES CORRESPON AMB L'USUARI ACTUAL, AFEGIM LA ICONA PER PODER ELIMINAR EL POST
                    if (!deleteButtonAdded && userId.equals(mAuth.getCurrentUser().getUid())) {
                        ImageButton viewPost_delete_imageButton = new ImageButton(getApplicationContext());
                        viewPost_delete_imageButton.setImageResource(R.drawable.ic_delete_black_35dp);
                        viewPost_delete_imageButton.setBackgroundColor(getResources().getColor(android.R.color.transparent));
                        viewPost_delete_imageButton.setOnClickListener(new View.OnClickListener() {
                            @Override
                            public void onClick(View v) {
                                viewPost_deletePostButtonClicked();
                            }
                        });
                        viewPost_icons_linearLayout.addView(viewPost_delete_imageButton);
                        deleteButtonAdded = true;
                    }
                } else {
                    //EL POST HA SIGUT ELIMINAT, FINALITZEM LA ACTIVITY
                    finish();
                }

            }

            @Override
            public void onCancelled(DatabaseError databaseError) {

            }
        };

        mDatabase.addValueEventListener(postListener);

        //LISTENER PER USUARIS QUE HAN FET LIKE
        ValueEventListener userLikesListener = new ValueEventListener() {
            @Override
            public void onDataChange(DataSnapshot dataSnapshot) {
                if (dataSnapshot.hasChild(mAuth.getCurrentUser().getUid())) {
                    // L'USUARI HA FET LIKE, PINTEM LA ICONA DE VERMELL
                    viewPost_favIcon_imageButton = findViewById(R.id.viewPost_favIcon_imageButton);
                    viewPost_favIcon_imageButton.setColorFilter(ContextCompat.getColor(getApplicationContext(), android.R.color.holo_red_dark), PorterDuff.Mode.SRC_IN);
                } else {
                    //L'USUARI NO HA FET LIKE, PINTEM LA ICONA DE NEGRE
                    viewPost_favIcon_imageButton = findViewById(R.id.viewPost_favIcon_imageButton);
                    viewPost_favIcon_imageButton.setColorFilter(ContextCompat.getColor(getApplicationContext(), android.R.color.black), PorterDuff.Mode.SRC_IN);
                }
            }

            @Override
            public void onCancelled(DatabaseError databaseError) {

            }
        };

        mDatabase.child("userLikes").addListenerForSingleValueEvent(userLikesListener);

        //RECUPEREM ELS COMENTARIS DE LA BASE DE DADES AMB EL MODEL COMMENTMODEL I OMPLIM LES VISTES ITEM_COMMENT MITJANÇANT LA CLASSE COMMENTITEMVIEWHOLDER
        final FirebaseRecyclerAdapter<CommentModel, ViewPostActivity.CommentItemViewHolder> FBRA = new FirebaseRecyclerAdapter<CommentModel, ViewPostActivity.CommentItemViewHolder>(CommentModel.class, R.layout.item_comment, ViewPostActivity.CommentItemViewHolder.class, mDatabaseComment) {
            @Override
            protected void populateViewHolder(final ViewPostActivity.CommentItemViewHolder viewHolder, CommentModel model, int position) {

                mDatabaseUsers.child(model.getUserId()).addValueEventListener(new ValueEventListener() {
                    @Override
                    public void onDataChange(DataSnapshot dataSnapshot) {
                        UserModel user = dataSnapshot.getValue(UserModel.class);
                        viewHolder.setUserName(user.getName());
                        viewHolder.setUserImage(getApplicationContext(), user.getImage());
                    }

                    @Override
                    public void onCancelled(DatabaseError databaseError) {

                    }
                });

                viewHolder.setComment(model.getComment());
                viewHolder.setDatetime(model.getTimestamp());
            }
        };

        mCommentsList.setAdapter(FBRA);
    }

    public void viewPost_imageClicked(View view){
        Intent viewPost_imageFullScreen = new Intent(ViewPostActivity.this, ImageFullScreenActivity.class);
        viewPost_imageFullScreen.putExtra("post_fireBaseKey", post_fireBaseKey);
        startActivity(viewPost_imageFullScreen);
    }

    // SALTEM A LA ACTIVITY COMMENT PASSANT LA CLAU DEL POST
    public void viewPost_commentButtonClicked(View view) {
        Intent viewPost_comment_intent = new Intent(ViewPostActivity.this, CommentActivity.class);
        viewPost_comment_intent.putExtra("post_fireBaseKey", post_fireBaseKey);
        startActivity(viewPost_comment_intent);
    }

    //FUNCIO PER FER LIKE
    public void viewPost_favButtonClicked(View view) {
        //RECUPEREM ELS USUARIS QUE HAN FET LIKE
        mDatabase.child("userLikes").addListenerForSingleValueEvent(new ValueEventListener() {
            @Override
            public void onDataChange(DataSnapshot dataSnapshot) {
                if (dataSnapshot.hasChild(mAuth.getCurrentUser().getUid())) {
                    //L'USUARI ACTUAL ES TROBAVA ENTRE ELS USUARIS, PER TANT ELIMINEM EL NODE I PINTEM LA ICONA DE NEGRE
                    mDatabase.child("userLikes").child(mAuth.getCurrentUser().getUid()).removeValue();
                    viewPost_favIcon_imageButton = findViewById(R.id.viewPost_favIcon_imageButton);
                    viewPost_favIcon_imageButton.setColorFilter(ContextCompat.getColor(getApplicationContext(), android.R.color.black), PorterDuff.Mode.SRC_IN);
                } else {
                    //L'USUARI HA FET LIKE, AFEGIM EL NODE I PINTEM LA ICONA DE VERMELL
                    mDatabase.child("userLikes").child(mAuth.getCurrentUser().getUid()).setValue(true);
                    viewPost_favIcon_imageButton = findViewById(R.id.viewPost_favIcon_imageButton);
                    viewPost_favIcon_imageButton.setColorFilter(ContextCompat.getColor(getApplicationContext(), android.R.color.holo_red_dark), PorterDuff.Mode.SRC_IN);
                }
            }

            @Override
            public void onCancelled(DatabaseError databaseError) {

            }
        });
    }

    //FUNCIO PER ELIMINAR UNA PUBLICACIO
    public void viewPost_deletePostButtonClicked() {
        new AlertDialog.Builder(this)
                .setIcon(android.R.drawable.ic_dialog_alert)
                .setTitle(getResources().getString(R.string.Delete_post))
                .setMessage(getResources().getString(R.string.Confirm_delete_post))
                .setPositiveButton(getResources().getString(R.string.Yes), new DialogInterface.OnClickListener() {
                    @Override
                    public void onClick(DialogInterface dialog, int which) {
                        //L'USUARI HA CONFIRMAT LA ELIMINACIO DE LA PUBLICACIO, ESBORREM ELS NODES DE LA PUBLICACIO, TANT DE LA ARREL COM DEL LLISTAT DE CLAUS DE L'ESDEVENIMENT I DE L'USUARI
                        if (!TextUtils.isEmpty(eventId) && !TextUtils.isEmpty(userId)) {
                            FirebaseDatabase.getInstance().getReference().child("Events").child(eventId).child("posts").child(post_fireBaseKey).removeValue();
                            FirebaseDatabase.getInstance().getReference().child("Users").child(mAuth.getCurrentUser().getUid()).child("posts").child(post_fireBaseKey).removeValue();
                            mDatabase.removeValue();
                        }
                    }

                })
                .setNegativeButton(getResources().getString(R.string.No), null)
                .show();
    }

    //CLASSE AUXILIAR VIEWHOLDER PER EMPLENAR ELS LAYOUTS ITEM_COMMENT
    public static class CommentItemViewHolder extends RecyclerView.ViewHolder {
        public CommentItemViewHolder(View itemView) {
            super(itemView);
            View mView = itemView;
        }

        public void setUserName(String userName) {
            TextView itemComment_username_textView = itemView.findViewById(R.id.itemComment_userName_textView);
            itemComment_username_textView.setText(userName);
        }

        public void setComment(String comment) {
            TextView itemComment_comment_textView = itemView.findViewById(R.id.itemComment_comment_textView);
            itemComment_comment_textView.setText(comment);
        }

        public void setUserImage(Context ctx, String userImage) {
            ImageView itemComment_userImage_imageView = itemView.findViewById(R.id.itemComment_userImage_imageView);
            Picasso.with(ctx).load(userImage).into(itemComment_userImage_imageView);
        }

        public void setDatetime(long timestamp) {
            SimpleDateFormat sfd = new SimpleDateFormat("dd/MM/yyyy HH:mm");
            String dateTimeString = sfd.format(new Date(timestamp)).toString();
            TextView itemComment_datetime_textView = itemView.findViewById(R.id.itemComment_datetime_textView);
            itemComment_datetime_textView.setText(dateTimeString);
        }
    }
}

