package com.dartmaker.uponevent;

import android.content.Context;
import android.content.Intent;
import android.graphics.drawable.Drawable;
import android.support.annotation.ColorRes;
import android.support.annotation.NonNull;
import android.support.design.widget.BottomNavigationView;
import android.support.design.widget.CollapsingToolbarLayout;
import android.support.v4.graphics.drawable.DrawableCompat;
import android.support.v7.app.AppCompatActivity;
import android.os.Bundle;
import android.support.v7.widget.LinearLayoutManager;
import android.support.v7.widget.RecyclerView;
import android.support.v7.widget.Toolbar;
import android.view.LayoutInflater;
import android.view.Menu;
import android.view.MenuItem;
import android.view.View;
import android.view.ViewGroup;
import android.widget.ImageView;
import android.widget.TextView;

import com.firebase.ui.database.FirebaseRecyclerAdapter;
import com.google.firebase.auth.FirebaseAuth;
import com.google.firebase.database.DataSnapshot;
import com.google.firebase.database.DatabaseError;
import com.google.firebase.database.DatabaseReference;
import com.google.firebase.database.FirebaseDatabase;
import com.google.firebase.database.ValueEventListener;
import com.squareup.picasso.Picasso;

import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;
import java.util.List;

public class ViewUserActivity extends AppCompatActivity {

    private String user_fireBaseKey;
    private DatabaseReference mDatabase;
    private DatabaseReference mUsersDatabase;
    private DatabaseReference mDatabase_userPosts;
    private DatabaseReference mDataBase_posts;
    private FirebaseAuth mAuth;
    private Menu menu;
    private int position;

    private ImageView viewUser_userImage;
    private RecyclerView mPostsList;
    private LinearLayoutManager layoutManager;
    private CollapsingToolbarLayout collapsingToolbar;
    private BottomNavigationView bottomNavigation;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_view_user);

        //RECUPEREM LES REFERENCIES PRINCIPALS
        Intent intent = getIntent();
        user_fireBaseKey = intent.getStringExtra("user_fireBaseKey");
        mDatabase = FirebaseDatabase.getInstance().getReference().child("Users").child(user_fireBaseKey);
        mUsersDatabase = FirebaseDatabase.getInstance().getReference().child("Users");
        position = 0;

        mDataBase_posts = FirebaseDatabase.getInstance().getReference().child("Posts");
        mAuth = FirebaseAuth.getInstance();
        final Toolbar toolbar = findViewById(R.id.toolbar);
        setSupportActionBar(toolbar);
        getSupportActionBar().setDisplayHomeAsUpEnabled(false);

        //Elements de la capcelera de pantalla
        viewUser_userImage = findViewById(R.id.viewUser_userImage);
        bottomNavigation = findViewById(R.id.viewUser_bottom_navigation);

        //Recuperem la recyclerview on anira el llistat de publicacions
        mPostsList = findViewById(R.id.viewUser_postList_recyclerView);
        layoutManager = new LinearLayoutManager(this);
        mPostsList.setLayoutManager(layoutManager);
        mPostsList.setHasFixedSize(true);
        mPostsList.setNestedScrollingEnabled(true);

        collapsingToolbar = findViewById(R.id.viewUser_collapsingToolbar_collapsingToolbarLayout);
        collapsingToolbar.setContentScrimColor(getResources().getColor(R.color.colorPrimary));
        collapsingToolbar.setStatusBarScrimColor(getResources().getColor(R.color.cardview_light_background));
    }


    @Override
    protected void onStart() {
        super.onStart();

        //CREAM EL MENU HORITZONTAL DE NAVEGACIO PER LA PANTALLA VIEWUSER
        bottomNavigation.setSelectedItemId(R.id.menuMain_profile_item);
        bottomNavigation.setOnNavigationItemSelectedListener(new BottomNavigationView.OnNavigationItemSelectedListener() {
            @Override
            public boolean onNavigationItemSelected(@NonNull MenuItem item) {
                int id = item.getItemId();

                if (id == R.id.menuMain_home_item) {
                    Intent intent = new Intent(ViewUserActivity.this, MainActivity.class);
                    startActivity(intent);
                    finish();
                } else if (id == R.id.addIcon) {
                    Intent intent = new Intent(ViewUserActivity.this, NewEventActivity.class);
                    startActivity(intent);
                    finish();
                } else if (id == R.id.menuMain_profile_item) {
                    return true;
                } else if (id == R.id.menuMain_search_item) {
                    Intent main_search_intent = new Intent(ViewUserActivity.this, SearchActivity.class);
                    startActivity(main_search_intent);
                    finish();
                }

                return true;
            }
        });

        //LISTENER DE L'USUARI
        ValueEventListener eventListener = new ValueEventListener() {
            @Override
            public void onDataChange(DataSnapshot dataSnapshot) {
                //EN PRIMER LLOC RECUPEREM LES DADES DE L'USUARI I OMPLIM LA INFORMACIO DE LA CAPÇALERA
                final UserModel user = dataSnapshot.getValue(UserModel.class);
                collapsingToolbar.setTitle(user.getName());
                collapsingToolbar.setOnClickListener(new View.OnClickListener() {
                    @Override
                    public void onClick(View v) {
                        Intent viewUser_imageFullScreen = new Intent(ViewUserActivity.this, ImageFullScreenActivity.class);
                        viewUser_imageFullScreen.putExtra("user_fireBaseKey", user_fireBaseKey);
                        startActivity(viewUser_imageFullScreen);
                    }
                });
                Picasso.with(getApplicationContext()).load(user.getImage()).into(viewUser_userImage);

                //COMPROVEM SI L'USUARI TE PUBLICACIONS A TRAVES DEL NODE POSTS DE DINTRE DE L'USUARI (LLISTAT DE CLAUS AMB BOLEANS)
                if (dataSnapshot.hasChild("posts")) {
                    mDatabase_userPosts = mDatabase.child("posts");

                    //FUNCIO PER OMPLIR EL RECYCLER AMB ELS ESDEVENIMENTS
                    readPosts(new ViewEventActivity.GetPostsCallback() {
                        @Override
                        public void onCallback(List<HashMap<String,Boolean>> posts) {
                            ViewUserActivity.PostsAdapter adapter = new ViewUserActivity.PostsAdapter(posts);
                            mPostsList.setAdapter(adapter);
                            mPostsList.scrollToPosition(position);
                        }
                    });
                }
            }

            @Override
            public void onCancelled(DatabaseError databaseError) {

            }
        };

        mDatabase.addValueEventListener(eventListener);

    }

    //FUNCIO VIEWHOLDER AUXILIAR PER EMPLENAR ELS LAYOUTS ITEM_POST
    public static class PostItemViewHolder_ extends RecyclerView.ViewHolder {
        public PostItemViewHolder_(View itemView) {
            super(itemView);
            View mView = itemView;
        }

        public void setText(String text) {
            TextView post_text = itemView.findViewById(R.id.itemPost_text_textView);
            post_text.setText(text);
        }

        public void setImage(Context ctx, String image) {
            ImageView event_image = itemView.findViewById(R.id.itemPost_image_imageView);
            Picasso.with(ctx).load(image).into(event_image);
        }

        public void setUserName(String userName) {
            TextView event_userName = itemView.findViewById(R.id.itemPost_userName_textView);
            event_userName.setText(userName);
        }

        public void setUserImage(Context ctx, String userImage) {
            ImageView event_userImage = itemView.findViewById(R.id.itemPost_userImage_imageView);
            Picasso.with(ctx).load(userImage).into(event_userImage);
        }

        public void setDatetime(long timestamp) {
            SimpleDateFormat sfd = new SimpleDateFormat("dd/MM/yyyy HH:mm");
            String dateTimeString = sfd.format(new Date(timestamp)).toString();
            TextView itemPost_datetime_textView = itemView.findViewById(R.id.itemPost_timestamp_textView);
            itemPost_datetime_textView.setText(dateTimeString);
        }

        public void setNumLikes(long numLikes, String literal) {
            TextView post_numLikes = itemView.findViewById(R.id.itemPost_numLikes_textView);
            if (numLikes == 0) {
                post_numLikes.setVisibility(View.INVISIBLE);
                post_numLikes.setHeight(0);
            } else {
                post_numLikes.setText(String.valueOf(numLikes) + " " + literal);
            }
        }

        public void setUserNameBold(String userName) {
            TextView post_userNameBold = itemView.findViewById(R.id.itemPost_userNameBold_textView);
            post_userNameBold.setText(userName + ": ");
        }

        public void setDateDiff(long start, long end) {
            TextView post_dateDiff = itemView.findViewById(R.id.itemPost_dateDiff_textView);
            String difference = ApplicationUtils.getDateDiff(start, end);
            post_dateDiff.setText(difference);
        }

        public void setNumComments(long numComments, String startLiteral, String endLiteral) {
            TextView post_numComments = itemView.findViewById(R.id.itemPost_numComments_textView);
            if (numComments == 0) {
                post_numComments.setVisibility(View.INVISIBLE);
                post_numComments.setHeight(0);
            }
            post_numComments.setText(startLiteral + " " + String.valueOf(numComments) + " " + endLiteral);
        }
    }

    //FUNCIO PER GENERAR EL MENU PROPI DE LA ACTIVITY VIEW USER
    @Override
    public boolean onCreateOptionsMenu(Menu menu) {
        this.menu = menu;
        getMenuInflater().inflate(R.menu.menu_view_user, menu);
        //SI L'USUARI ES CORRESPON AMB L'USUARI ACTUAL, VOL DIR QUE HEM ENTRAT AL NOSTRE PERFIL, AFEGIM ICONES DE EDICIO I DE TANCAR SESSIO
        if (user_fireBaseKey.equals(mAuth.getCurrentUser().getUid())) {
            Intent viewUser_editUser_intent = new Intent(ViewUserActivity.this, EditUserActivity.class);
            menu.add(Menu.NONE, Menu.NONE, Menu.NONE, "Edit").setIcon(R.drawable.ic_edit_white_35dp).setIntent(viewUser_editUser_intent).setShowAsAction(MenuItem.SHOW_AS_ACTION_ALWAYS);
            menu.add(Menu.NONE, Menu.NONE, Menu.NONE, "LogOut").setIcon(R.drawable.ic_exit_to_app_white_35dp).setOnMenuItemClickListener(new MenuItem.OnMenuItemClickListener() {
                @Override
                public boolean onMenuItemClick(MenuItem item) {
                    mAuth.signOut();
                    return true;
                }
            }).setShowAsAction(MenuItem.SHOW_AS_ACTION_ALWAYS);
        }

        final MenuItem menuViewUser_follow_item = menu.findItem(R.id.menuViewUser_followUser_item);
        if (menuViewUser_follow_item != null) {
            if (mAuth.getCurrentUser().getUid().equals(user_fireBaseKey)) {
                menuViewUser_follow_item.setVisible(false);
            }
            //RECUPEREM EL LLISTAT DE FOLLOWS
            mUsersDatabase.child(mAuth.getCurrentUser().getUid()).child("usersFollowed").addListenerForSingleValueEvent(new ValueEventListener() {
                @Override
                public void onDataChange(DataSnapshot dataSnapshot) {
                    if (dataSnapshot.hasChild(user_fireBaseKey)) {
                        //SI L'USUARI ACTUAL HA FET LIKE, PINTEM LA ICONA DE VERMELL
                        tintMenuIcon(ViewUserActivity.this, menuViewUser_follow_item, R.color.gold);
                    } else {
                        // SI L'USUARI ACTUAL NO HA FET LIKE, PINTEM LA ICONA DE BLANC
                        tintMenuIcon(ViewUserActivity.this, menuViewUser_follow_item, android.R.color.white);
                    }
                }

                @Override
                public void onCancelled(DatabaseError databaseError) {

                }
            });

        }

        return true;
    }

    //FUNCIONALITATS DE AFEGIR USUARI COM AMIC I VEURE INFO DE USUARI NO IMPLEMENTADES
    @Override
    public boolean onOptionsItemSelected(MenuItem item) {
        switch (item.getItemId()) {

            case R.id.menuViewUser_followUser_item:
                viewUser_followButtonClicked();
                return true;

            case R.id.menuViewUser_userInfo_item:
                Intent viewUser_viewUserInfo = new Intent(ViewUserActivity.this, ViewUserInfoActivity.class);
                viewUser_viewUserInfo.putExtra("user_fireBaseKey", user_fireBaseKey);
                startActivity(viewUser_viewUserInfo);
                return true;
        }
        return super.onOptionsItemSelected(item);
    }

    //FUNCIO AUXILIAT PER CANVIAR UN ITEM DEL MENU DE COLOR
    public static void tintMenuIcon(Context context, MenuItem item, @ColorRes int color) {
        Drawable normalDrawable = item.getIcon();
        Drawable wrapDrawable = DrawableCompat.wrap(normalDrawable);
        DrawableCompat.setTint(wrapDrawable, context.getResources().getColor(color));

        item.setIcon(wrapDrawable);
    }

    // INDIQUEM L'USUARI COM A FOLLOW
    public void viewUser_followButtonClicked() {
        //LISTENER DE USUARIS A QUI S'HA FET FOLLOW
        mUsersDatabase.child(mAuth.getCurrentUser().getUid()).child("usersFollowed").addListenerForSingleValueEvent(new ValueEventListener() {
            @Override
            public void onDataChange(DataSnapshot dataSnapshot) {
                //SI JA ERA FAV, ELIMINEM EL NODE I CANVIEM LA ICONA A BLANC
                if (dataSnapshot.hasChild(user_fireBaseKey)) {
                    mUsersDatabase.child(mAuth.getCurrentUser().getUid()).child("usersFollowed").child(user_fireBaseKey).removeValue();
                    tintMenuIcon(ViewUserActivity.this, menu.findItem(R.id.menuViewUser_followUser_item), android.R.color.white);
                } else {
                    //NO ERA FAV, AFEGIM EL NODE AMB LA CLAU D'USUARI I CANVIEM LA ICONA A VERMELL
                    mUsersDatabase.child(mAuth.getCurrentUser().getUid()).child("usersFollowed").child(user_fireBaseKey).setValue(true);
                    tintMenuIcon(ViewUserActivity.this, menu.findItem(R.id.menuViewUser_followUser_item), R.color.gold);
                }
            }

            @Override
            public void onCancelled(DatabaseError databaseError) {

            }
        });
    }

    //ADAPTER NECESSARI PER EL VIEWHOLDER
    class PostsAdapter extends RecyclerView.Adapter {

        private List<HashMap<String, Boolean>> models = new ArrayList<>();

        public PostsAdapter(final List<HashMap<String, Boolean>> viewModels) {
            if (viewModels != null) {
                this.models.addAll(viewModels);
            }
        }

        @Override
        public RecyclerView.ViewHolder onCreateViewHolder(final ViewGroup parent, final int viewType) {
            final View view = LayoutInflater.from(parent.getContext()).inflate(viewType, parent, false);
            return new ViewUserActivity.PostItemViewHolder(view);
        }

        @Override
        public void onBindViewHolder(final RecyclerView.ViewHolder holder, final int position) {
            ((ViewUserActivity.PostItemViewHolder) holder).bindData(models.get(position));
        }

        @Override
        public int getItemCount() {
            return models.size();
        }


        @Override
        public int getItemViewType(final int position) {
            return R.layout.item_post;
        }
    }

    //CLASSE VIEWHOLDER PER EMPLENAR ELS LAYOUTS ITEM_EVENT
    public class PostItemViewHolder extends RecyclerView.ViewHolder {

        TextView post_text = itemView.findViewById(R.id.itemPost_text_textView);
        ImageView post_image = itemView.findViewById(R.id.itemPost_image_imageView);
        TextView post_userName = itemView.findViewById(R.id.itemPost_userName_textView);
        ImageView post_userImage = itemView.findViewById(R.id.itemPost_userImage_imageView);
        TextView itemPost_datetime_textView = itemView.findViewById(R.id.itemPost_timestamp_textView);
        TextView post_numLikes = itemView.findViewById(R.id.itemPost_numLikes_textView);
        TextView post_userNameBold = itemView.findViewById(R.id.itemPost_userNameBold_textView);
        TextView post_dateDiff = itemView.findViewById(R.id.itemPost_dateDiff_textView);
        TextView post_numComments = itemView.findViewById(R.id.itemPost_numComments_textView);

        public PostItemViewHolder(View itemView) {
            super(itemView);
            View mView = itemView;
        }

        public void bindData(final HashMap<String, Boolean> post) {

            final String post_fireBaseKey = post.keySet().iterator().next();

            mDataBase_posts.child(post_fireBaseKey).addValueEventListener(new ValueEventListener() {
                @Override
                public void onDataChange(DataSnapshot dataSnapshot) {
                    PostModel post = dataSnapshot.getValue(PostModel.class);

                    //COMPROVEM QUE NO HAGI SIGUT ELIMINAT
                    if (post != null) {

                        //EMPLENEM ELS LAYOUTS A TRAVES DE LA CLASSE VIEWHOLDER EMPRANT EL RECYCLERADAPTER CREAT AMB UN MODEL DE TIPUS BOLEA
                        final String userId = post.getUserId();
                        post_text.setText(post.getText());
                        Picasso.with(getApplicationContext()).load(post.getImage()).into(post_image);

                        SimpleDateFormat sfd = new SimpleDateFormat("dd/MM/yyyy HH:mm");
                        String dateTimeString = sfd.format(new Date(post.getTimestamp())).toString();
                        itemPost_datetime_textView.setText(dateTimeString);

                        long numComments = dataSnapshot.child("comments").getChildrenCount();
                        if (numComments == 0) {
                            post_numComments.setVisibility(View.INVISIBLE);
                            post_numComments.setHeight(0);
                        } else {
                            post_numComments.setText(getResources().getString(R.string.Read_the) + " " + String.valueOf(numComments) + " " + getResources().getString(R.string.Comments));
                        }

                        long numLikes = dataSnapshot.child("userLikes").getChildrenCount();
                        if (numLikes == 0) {
                            post_numLikes.setVisibility(View.INVISIBLE);
                            post_numLikes.setHeight(0);
                        } else {

                            post_numLikes.setText(String.valueOf(numLikes) + " " + getResources().getString(R.string.Like));
                        }

                        String difference = ApplicationUtils.getDateDiff(post.getTimestamp(), System.currentTimeMillis());
                        post_dateDiff.setText(difference);

                        itemView.findViewById(R.id.itemPost_image_imageView).setOnClickListener(new View.OnClickListener() {
                            @Override
                            public void onClick(View v) {
                                position = getAdapterPosition();
                                Intent viewEvent_viewPost_Intent = new Intent(ViewUserActivity.this, ViewPostActivity.class);
                                viewEvent_viewPost_Intent.putExtra("post_fireBaseKey", post_fireBaseKey);
                                startActivity(viewEvent_viewPost_Intent);
                            }
                        });

                        itemView.findViewById(R.id.itemPost_text_linearLayout).setOnClickListener(new View.OnClickListener() {
                            @Override
                            public void onClick(View v) {
                                position = getAdapterPosition();
                                Intent viewEvent_viewPost_Intent = new Intent(ViewUserActivity.this, ViewPostActivity.class);
                                viewEvent_viewPost_Intent.putExtra("post_fireBaseKey", post_fireBaseKey);
                                startActivity(viewEvent_viewPost_Intent);
                            }
                        });

                        itemView.findViewById(R.id.itemPost_numComments_textView).setOnClickListener(new View.OnClickListener() {
                            @Override
                            public void onClick(View v) {
                                position = getAdapterPosition();
                                Intent itemPost_comment_intent = new Intent(ViewUserActivity.this, CommentActivity.class);
                                itemPost_comment_intent.putExtra("post_fireBaseKey", post_fireBaseKey);
                                startActivity(itemPost_comment_intent);
                            }
                        });

                        itemView.findViewById(R.id.itemPost_numLikes_textView).setOnClickListener(new View.OnClickListener() {
                            @Override
                            public void onClick(View v) {
                                position = getAdapterPosition();
                                Intent itemPost_userList_intent = new Intent(ViewUserActivity.this, UserListActivity.class);
                                itemPost_userList_intent.putExtra("post_fireBaseKey", post_fireBaseKey);
                                startActivity(itemPost_userList_intent);
                            }
                        });


                        mUsersDatabase.child(post.getUserId()).addValueEventListener(new ValueEventListener() {
                            @Override
                            public void onDataChange(DataSnapshot dataSnapshot) {
                                //EMPLENEM ELS CAMPS D'INFORMACIO D'USUARI DEL LAYOUT ITEM_POST
                                UserModel user = dataSnapshot.getValue(UserModel.class);

                                post_userName.setText(user.getName());
                                post_userNameBold.setText(user.getUsername() + ": ");

                                Picasso.with(getApplicationContext()).load(user.getImage()).into(post_userImage);

                            }

                            @Override
                            public void onCancelled(DatabaseError databaseError) {

                            }
                        });
                    }
                }

                @Override
                public void onCancelled(DatabaseError databaseError) {

                }
            });

        }
    }

    //CALLBACK PER PODER EMPLENAR ELS ESDEVENIMENTS UN COP CARREGATS
    public interface GetPostsCallback {
        void onCallback(List<HashMap<String, Boolean>> posts);
    }

    //FUNCIO QUE RECORRE ELS ESDEVENIMENTS I LES RETORNA AL CALLBACK
    public void readPosts(final ViewEventActivity.GetPostsCallback getPostsCallback) {
        mDatabase_userPosts.addListenerForSingleValueEvent(new ValueEventListener() {

            @Override
            public void onDataChange(DataSnapshot dataSnapshot) {
                List<HashMap<String, Boolean>> posts = new ArrayList<>();
                for (DataSnapshot snapshot : dataSnapshot.getChildren()) {
                    String postKey = snapshot.getKey();

                    Boolean post = snapshot.getValue(Boolean.class);
                    HashMap<String, Boolean> hashMap = new HashMap<>();
                    hashMap.put(postKey, post);
                    posts.add(hashMap);
                }
                getPostsCallback.onCallback(posts);

            }

            @Override
            public void onCancelled(DatabaseError databaseError) {

            }
        });
    }
}

