package com.dartmaker.uponevent;

import android.content.Context;
import android.content.Intent;
import android.graphics.Color;
import android.support.design.widget.CollapsingToolbarLayout;
import android.support.v7.app.AppCompatActivity;
import android.os.Bundle;
import android.support.v7.widget.LinearLayoutManager;
import android.support.v7.widget.RecyclerView;
import android.support.v7.widget.Toolbar;
import android.view.Menu;
import android.view.MenuItem;
import android.view.View;
import android.webkit.WebChromeClient;
import android.webkit.WebView;
import android.widget.ImageView;
import android.widget.TextView;

import com.firebase.ui.database.FirebaseRecyclerAdapter;
import com.google.firebase.auth.FirebaseAuth;
import com.google.firebase.database.DataSnapshot;
import com.google.firebase.database.DatabaseError;
import com.google.firebase.database.DatabaseReference;
import com.google.firebase.database.FirebaseDatabase;
import com.google.firebase.database.ValueEventListener;
import com.squareup.picasso.Picasso;

import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.Locale;

public class ViewUserInfoActivity extends AppCompatActivity {

    private String user_fireBaseKey;
    private DatabaseReference mDatabase;
    private DatabaseReference mDatabase_userPosts;
    private DatabaseReference mDatabasePosts;
    private FirebaseAuth mAuth;
    private ApplicationUtils applicationUtils;

    private ImageView viewUserInfo_userImage;
    private TextView viewUserInfo_user_textView;
    private TextView viewUserInfo_location_textView;
    private TextView viewUserInfo_date_textView;
    private WebView viewUserInfo_desc_webView;
    private RecyclerView mImagesList;
    private LinearLayoutManager layoutManager;
    private CollapsingToolbarLayout collapsingToolbar;
    private Menu menu;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_view_user_info);

        //INICIALITZEM LES REFERENCIES PRINCIPALS
        Intent intent = getIntent();
        user_fireBaseKey = intent.getStringExtra("user_fireBaseKey");
        mDatabase = FirebaseDatabase.getInstance().getReference().child("Users").child(user_fireBaseKey);
        mDatabasePosts = FirebaseDatabase.getInstance().getReference().child("Posts");
        mAuth = FirebaseAuth.getInstance();
        applicationUtils = new ApplicationUtils();

        //INICIALITZEM LES VARIABLES
        final Toolbar toolbar = findViewById(R.id.viewUserInfo_toolbar_toolbar);
        setSupportActionBar(toolbar);
        getSupportActionBar().setDisplayHomeAsUpEnabled(false);
        collapsingToolbar = findViewById(R.id.viewUserInfo_collapsingToolbar_collapsingToolbarLayout);
        collapsingToolbar.setContentScrimColor(getResources().getColor(R.color.colorPrimary));
        collapsingToolbar.setStatusBarScrimColor(getResources().getColor(R.color.cardview_light_background));
        viewUserInfo_userImage = findViewById(R.id.viewUserInfo_userImage);
        viewUserInfo_user_textView = findViewById(R.id.viewUserInfo_user_textView);
        viewUserInfo_location_textView = findViewById(R.id.viewUserInfo_location_textView);
        viewUserInfo_date_textView = findViewById(R.id.viewUserInfo_date_textView);
        viewUserInfo_desc_webView = findViewById(R.id.viewUserInfo_desc_webView);

        //Recuperem la recyclerview on anira el llistat de publicacions
        mImagesList = findViewById(R.id.viewUserInfo_galley_recyclerView);
        layoutManager = new LinearLayoutManager(this, LinearLayoutManager.HORIZONTAL, false);
        mImagesList.setLayoutManager(layoutManager);
        mImagesList.setHasFixedSize(true);
        mImagesList.setNestedScrollingEnabled(true);
    }

    @Override
    protected void onStart() {
        super.onStart();

        //LISTENER DE L'USUARI
        ValueEventListener eventListener = new ValueEventListener() {
            @Override
            public void onDataChange(DataSnapshot dataSnapshot) {

                //EN PRIMER LLOC RECUPEREM LES DADES DE L'USUARI I OMPLIM LA INFORMACIO DE LA CAPÇALERA
                final UserModel user = dataSnapshot.getValue(UserModel.class);
                collapsingToolbar.setTitle(user.getUsername());
                collapsingToolbar.setOnClickListener(new View.OnClickListener() {
                    @Override
                    public void onClick(View v) {
                        Intent viewUserInfo_imageFullScreen = new Intent(ViewUserInfoActivity.this, ImageFullScreenActivity.class);
                        viewUserInfo_imageFullScreen.putExtra("user_fireBaseKey", user_fireBaseKey);
                        startActivity(viewUserInfo_imageFullScreen);
                    }
                });
                Picasso.with(getApplicationContext()).load(user.getImage()).into(viewUserInfo_userImage);
                viewUserInfo_location_textView.setText(user.getLocation());

                SimpleDateFormat sfd = new SimpleDateFormat("dd/MM/yyyy", Locale.FRANCE);
                String dateTimeString = sfd.format(new Date(user.getBirthday()));
                viewUserInfo_date_textView.setText(dateTimeString);

                String html_data = "<style>img{max-width:100%;height:auto;} iframe{width:100%;}</style> ";
                html_data += user.getState();
                viewUserInfo_desc_webView.getSettings().setBuiltInZoomControls(true);
                viewUserInfo_desc_webView.setBackgroundColor(Color.TRANSPARENT);
                viewUserInfo_desc_webView.setWebChromeClient(new WebChromeClient());
                viewUserInfo_desc_webView.loadData(html_data, "text/html; charset=UTF-8", null);

                viewUserInfo_user_textView.setText(user.getName() + " " + user.getSurname());


                //COMPROVEM SI L'USUARI TE PUBLICACIONS A TRAVES DEL NODE POSTS DE DINTRE DE L'USUARI (LLISTAT DE CLAUS AMB BOLEANS)
                if (dataSnapshot.hasChild("posts")) {
                    mDatabase_userPosts = mDatabase.child("posts");
                    //CREAM UN RECYCLERADAPTER AMB UN MODEL DE TIPUS BOOLEAN PER TAL DE RECORRER SOLAMENT AQUELLES PUBLICACIONS PRESENTS A L'USUARI I OMPLIR POSTERIORMENT ELS LAYOUTS ITEM_POST AMB LES CLAUS LLISTADES
                    final FirebaseRecyclerAdapter<Boolean, ViewUserInfoActivity.ImageItemViewHolder> FBRA = new FirebaseRecyclerAdapter<Boolean, ViewUserInfoActivity.ImageItemViewHolder>(Boolean.class, R.layout.item_image, ViewUserInfoActivity.ImageItemViewHolder.class, mDatabase_userPosts) {
                        @Override
                        protected void populateViewHolder(final ViewUserInfoActivity.ImageItemViewHolder viewHolder, Boolean model, int position) {

                            //RECUPEREM LA CLAU DE LA PUBLICACIO
                            final String post_fireBaseKey = getRef(position).getKey();

                            //LISTENER DE LA PUBLICACIO
                            mDatabasePosts.child(post_fireBaseKey).addValueEventListener(new ValueEventListener() {
                                @Override
                                public void onDataChange(DataSnapshot dataSnapshot) {
                                    PostModel post = dataSnapshot.getValue(PostModel.class);

                                    //COMPROVEM QUE NO HAGI SIGUT ELIMINAT
                                    if (post != null) {
                                        //EMPLENEM ELS LAYOUTS A TRAVES DE LA CLASSE VIEWHOLDER EMPRANT EL RECYCLERADAPTER CREAT AMB UN MODEL DE TIPUS BOLEA
                                        viewHolder.setImage(getApplicationContext(), post.getImage());
                                        viewHolder.itemView.findViewById(R.id.itemImage_image_imageView).setOnClickListener(new View.OnClickListener() {
                                            @Override
                                            public void onClick(View v) {
                                                Intent viewUserInfo_imageFullScreen_Intent = new Intent(ViewUserInfoActivity.this, ImageFullScreenActivity.class);
                                                viewUserInfo_imageFullScreen_Intent.putExtra("post_fireBaseKey", post_fireBaseKey);
                                                startActivity(viewUserInfo_imageFullScreen_Intent);
                                            }
                                        });
                                    }
                                }

                                @Override
                                public void onCancelled(DatabaseError databaseError) {

                                }
                            });
                        }
                    };

                    mImagesList.setAdapter(FBRA);
                }
            }

            @Override
            public void onCancelled(DatabaseError databaseError) {

            }
        };

        mDatabase.addValueEventListener(eventListener);

    }

    //FUNCIO VIEWHOLDER AUXILIAR PER EMPLENAR ELS LAYOUTS ITEM_IMAGE
    public static class ImageItemViewHolder extends RecyclerView.ViewHolder {
        public ImageItemViewHolder(View itemView) {
            super(itemView);
            View mView = itemView;
        }

        public void setImage(Context ctx, String image) {
            ImageView postImage = itemView.findViewById(R.id.itemImage_image_imageView);
            Picasso.with(ctx).load(image).into(postImage);

        }
    }

    //FUNCIO PER GENERAR EL MENU PROPI DE LA ACTIVITY VIEW USER INFO
    @Override
    public boolean onCreateOptionsMenu(final Menu menu) {
        this.menu = menu;

        //SI L'USUARI ES CORRESPON AMB L'USUARI ACTUAL, TINDRA PERMISOS D'EDICIO, CREAM LA ICONA
        if(user_fireBaseKey.equals(mAuth.getCurrentUser().getUid())){

            Intent viewUserInfo_editUser_intent = new Intent(ViewUserInfoActivity.this, EditUserActivity.class);
            viewUserInfo_editUser_intent.putExtra("user_fireBaseKey", user_fireBaseKey);
            menu.add(Menu.NONE,Menu.NONE, Menu.NONE, "Edit").setIcon(R.drawable.ic_edit_white_35dp).setIntent(viewUserInfo_editUser_intent).setShowAsAction(MenuItem.SHOW_AS_ACTION_ALWAYS);
        }

        //CREEM EL MENU
        getMenuInflater().inflate(R.menu.menu_view_user_info, menu);

        return true;
    }
}
